<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Api\Exception;

class ApiRequestException extends \Exception
{
    const ERROR_NAME_TRANSACTION_ABORT = 'TRANSACTION_ABORTED';
    const ERROR_NAME_TRANSACTION_ALREADY_CAPTURED = 'TRANSACTION_ALREADY_CAPTURED';
    const ERROR_NAME_TRANSACTION_DECLINED = 'TRANSACTION_DECLINED';
    const ERROR_NAME_INTERNAL_ERROR = 'INTERNAL_ERROR';
    const BEHAVIOR_RETRY_LATER = 'RETRY_LATER';

    private string $requestUri;
    private string $responseBody;
    private ?object $responseObject = null;

    /**
     * Initialize all the class parameters
     */
    public function __construct(string $requestUri, string $responseBody, int $code)
    {
        parent::__construct('Saferpay API Request Failed (URI: ' . $requestUri . '): ' . $responseBody, $code);

        $this->requestUri = $requestUri;
        $this->responseBody = $responseBody;

        $responseObject = @json_decode($responseBody);
        if (is_object($responseObject)) {
            $this->responseObject = $responseObject;
        }
    }

    /**
     * Get the value of requestUri
     */
    public function getRequestUri(): string
    {
        return $this->requestUri;
    }

    /**
     * Get the value of responseBody
     */
    public function getResponseBody(): string
    {
        return $this->responseBody;
    }

    /**
     * Get the name of the error
     */
    public function getErrorName(): string
    {
        if (
            $this->responseObject &&
            isset($this->responseObject->ErrorName) &&
            is_string($this->responseObject->ErrorName)
        ) {
            return $this->responseObject->ErrorName;
        }

        return '';
    }

    /**
     * Get the transaction id from the error
     */
    public function getTransactionId(): string
    {
        if ($this->responseObject && isset($this->responseObject->TransactionId)) {
            return $this->responseObject->TransactionId;
        }
        return '';
    }

    /**
     * Get behaviour of the error
     */
    public function getBehaviour(): string
    {
        if (
            $this->responseObject &&
            isset($this->responseObject->Behavior) &&
            is_string($this->responseObject->Behavior)
        ) {
            return $this->responseObject->Behavior;
        }

        return '';
    }

    /**
     * Get the value of responseObject
     */
    public function getResponseObject(): ?object
    {
        return $this->responseObject;
    }
}
