<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Controller;

use Exception;
use Saferpay\Shopware\Exception\SaferpayShopwareNotFoundException;
use Saferpay\Shopware\Service\PaymentService;
use Shopware\Core\Framework\Context;
use Shopware\Core\System\SalesChannel\SalesChannelEntity;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
//use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;

#[Route(defaults: ['_routeScope' => ['administration']])]
class AdminController extends AbstractController
{
    public PaymentService $paymentService;

    /**
     * Initialize the parameters of the class
     */
    public function __construct(
        PaymentService $paymentService
    ) {
        $this->paymentService = $paymentService;
    }

    /**
     * Manual Capture
     */
    #[Route(path: '/api/v{version}/_action/saferpay-shopware/manual-capture/{paymentTransactionId}',
    name: 'api.saferpay_shopware.manual_capture.get', methods: ['GET'])]
    public function manualCapture(string $paymentTransactionId, Request $request, Context $context): JsonResponse
    {
        try {
            $this->paymentService->manualCapture($paymentTransactionId, $context);
            return new JsonResponse(['success' => true]);
        } catch (Exception $exception) {
            return new JsonResponse(['success' => false]);
        }
    }

    /**
     * Manual Cancellation
     */
    #[Route(path: '/api/v{version}/_action/saferpay-shopware/manual-cancellation/{paymentTransactionId}',
    name: 'api.saferpay_shopware.manual_cancellation.get',methods: ['GET'])]
    public function cancelTransactions(string $paymentTransactionId, Context $context): JsonResponse
    {

        try {
            $this->paymentService->cancelPayments($paymentTransactionId, $context);
            return new JsonResponse(['success' => true]);
        } catch (\Throwable $th) {
            return new JsonResponse(['success' => false]);
        }
    }
}
