<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Core\Content\Saferpay;

use Shopware\Core\Checkout\Customer\CustomerDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\EntityDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\Required;
use Shopware\Core\Framework\DataAbstractionLayer\Field\BoolField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\DateTimeField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\FkField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\PrimaryKey;
use Shopware\Core\Framework\DataAbstractionLayer\Field\IdField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\IntField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\ManyToOneAssociationField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\StringField;
use Shopware\Core\Framework\DataAbstractionLayer\FieldCollection;

class SaferpayCustomerSecureTransactionDefinition extends EntityDefinition
{
    public const ENTITY_NAME = 'saferpay_customer_secure_transaction';

    public function getEntityName(): string
    {
        return self::ENTITY_NAME;
    }

    public function getCollectionClass(): string
    {
        return SaferpayCustomerSecureTransactionCollection::class;
    }

    public function getEntityClass(): string
    {
        return SaferpayCustomerSecureTransactionEntity::class;
    }

    protected function defineFields(): FieldCollection
    {
        return new FieldCollection([
            (new IdField('id', 'id'))->addFlags(new Required(), new PrimaryKey()),
            (new FkField('customer_id', 'customerId', CustomerDefinition::class)),
            (new StringField('saferpay_request_id', 'saferpayRequestId')),
            (new StringField('saferpay_token', 'saferpayToken')),
            (new StringField('saferpay_alias_id', 'saferpayAliasId')),
            (new IntField('saferpay_alias_lifetime', 'saferpayAliasLifetime')),
            (new StringField('saferpay_payment_method', 'saferpayPaymentMethod')),
            (new StringField('saferpay_display_text', 'saferpayDisplayText')),
            (new StringField('saferpay_masked_number', 'saferpayMaskedNumber')),
            (new StringField('saferpay_exp_year', 'saferpayExpYear')),
            (new StringField('saferpay_exp_month', 'saferpayExpMonth')),
            (new StringField('saferpay_holder_name', 'saferpayHolderName')),
            (new IntField('saferpay_customer_id', 'saferpayCustomerId')),
            (new StringField('saferpay_authenticated', 'saferpayAuthenticated')),
            (new StringField('live_transaction', 'liveTransaction')),
            (new StringField('alias_active', 'aliasActive')),
            (new DateTimeField('saferpay_created_at', 'saferpayCreatedAt')),

            new ManyToOneAssociationField('customer', 'customer_id', CustomerDefinition::class, 'id', false)
        ]);
    }
}
