<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Core\Content\Saferpay;

use Shopware\Core\Checkout\Customer\CustomerDefinition;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionDefinition;
use Shopware\Core\Checkout\Order\OrderDefinition;
use Shopware\Core\Checkout\Payment\PaymentMethodDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\EntityDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\Required;
use Shopware\Core\Framework\DataAbstractionLayer\Field\BoolField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\DateTimeField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\FkField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\PrimaryKey;
use Shopware\Core\Framework\DataAbstractionLayer\Field\FloatField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\IdField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\IntField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\JsonField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\ManyToOneAssociationField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\StringField;
use Shopware\Core\Framework\DataAbstractionLayer\FieldCollection;

class SaferpayPaymentTransactionDefinition extends EntityDefinition
{
    public const ENTITY_NAME = 'saferpay_payment_transaction';

    public function getEntityName(): string
    {
        return self::ENTITY_NAME;
    }

    public function getCollectionClass(): string
    {
        return SaferpayPaymentTransactionCollection::class;
    }

    public function getEntityClass(): string
    {
        return SaferpayPaymentTransactionEntity::class;
    }

    protected function defineFields(): FieldCollection
    {
        return new FieldCollection([
            (new IdField('id', 'id'))->addFlags(new Required(), new PrimaryKey()),
            (new FkField('payment_id', 'paymentId', PaymentMethodDefinition::class)),
            (new FkField('order_id', 'orderId', OrderDefinition::class)),
            (new FkField('order_transaction_id', 'orderTransactionId', OrderTransactionDefinition::class)),
            (new FkField('customer_id', 'customerId', CustomerDefinition::class)),
            (new StringField('saferpay_request_id', 'saferpayRequestId')),
            (new StringField('saferpay_token', 'saferpayToken')),
            (new DateTimeField('saferpay_token_expiry_date', 'saferpayTokenExpiryDate')),
            (new StringField('authorisation_method', 'authorisationMethod')),
            (new IntField('saferpay_customer_id', 'saferpayCustomerId')),
            (new StringField('transaction_type', 'transactionType')),
            (new FloatField('authorization_amount', 'authorizationAmount')),
            (new BoolField('transaction_mode', 'transactionMode')),
            (new StringField('currency_code', 'currencyCode')),
            (new StringField('language_code', 'languageCode')),
            (new FloatField('captured_amount', 'capturedAmount')),
            (new StringField('capture_id', 'captureId')),
            (new JsonField('saferpay_payment_info', 'saferpayPaymentInfo')),
            (new BoolField('liability_shift_status', 'liabilityShiftStatus')),
            (new StringField('liable_entity', 'liableEntity')),
            (new BoolField('three_ds_liability_shift', 'threeDsLiabilityShift')),
            (new BoolField('dcc_status', 'dccStatus')),
            (new JsonField('dcc_info', 'dccInfo')),
            (new BoolField('alias_register_status', 'aliasRegisterStatus')),
            (new BoolField('paid_status', 'paidStatus')),
            (new BoolField('transaction_status', 'transactionStatus')),
            (new BoolField('active', 'active')),
            (new BoolField('preauth', 'preauth')),
            (new BoolField('recurring', 'recurring')),
            (new StringField('saferpay_alias_id', 'saferpayAliasId')),
            (new StringField('fraud_review_status', 'fraudReviewStatus')),

            new ManyToOneAssociationField('payment', 'payment_id', PaymentMethodDefinition::class, 'id', false),
            new ManyToOneAssociationField('order', 'order_id', OrderDefinition::class, 'id', false),
            new ManyToOneAssociationField(
                'orderTransaction', 'order_transaction_id', OrderTransactionDefinition::class, 'id', false),
            new ManyToOneAssociationField('customer', 'customer_id', CustomerDefinition::class, 'id', false)
        ]);
    }
}
