<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Installer;

use Saferpay\Shopware\SaferpayShopware;
use Shopware\Core\Defaults;
use Shopware\Core\System\CustomField\CustomFieldTypes;

class InstallerHelper
{

    public const FIELDSET_ID_PAYMENT_METHOD = 'b68e764adf244001b5c642c66d31491c';
    public const COUNTRY_FIELDSET_ID_PAYMENT_METHOD = 'c0c3aa3c6727465291324294ef22e011';
    public const ORDER_TOTAL_FIELDSET_ID_PAYMENT_METHOD = 'd02545ed8f334c01a6a345ffcffdad8a';
    public const CUSTOMER_FIELDSET_ID_PAYMENT_METHOD = 'ed3a58f89ef54d98b75a7aa0106e2835';
    public const SPECIFIC_FIELDSET_ID_PAYMENT_METHOD = 'f8bcc1731e5d4482a18d19aa01916386';
    public const SAFERPAY_FIELDS_APIURL_TEXT = "Saferpay Fields API-URL";
    public const SAFERPAY_FIELDS_ACCESS_TOKEN = 'Saferpay Fields Access Token';
    public const SAFERPAY_FIELDS_JS_LIBRARY_URL = "Test JS JS-Library-URL";
    public const SAFERPAY_FIELDS_CSS_URL = "Css URL";

    public const SAFERPAY_CREDIT_CARD_THEME = 'Credit Card Theme';



    /**
     * Get currencies as options
     */
    protected function getCurrencyOptions()
    {
        $currencies = [
            'AED' => 'United Arab Emirates dirham (AED)', 'AFN' => 'Afghan afghani (AFN)',
            'ALL' => 'Albanian lek (ALL)', 'AMD' => 'Armenian dram (AMD)',
            'AOA' => 'Angolan kwanza (AOA)', 'ARS' => 'Argentine peso (ARS)',
            'AUD' => 'Australian dollar (AUD)', 'AWG' => 'Aruban florin (AWG)',
            'AZN' => 'Azerbaijani manat (AZN)', 'BAM' => 'Bosnia and Herzegovina convertible mark (BAM)',
            'BBD' => 'Barbados dollar (BBD)', 'BDT' => 'Bangladeshi taka (BDT)',
            'BGN' => 'Bulgarian lev (BGN)', 'BHD' => 'Bahraini dinar (BHD)',
            'BIF' => 'Burundian franc (BIF)', 'BMD' => 'Bermudian dollar (BMD)',
            'BND' => 'Brunei dollar (BND)', 'BOB' => 'Boliviano (BOB)',
            'BOV' => 'Bolivian Mvdol (BOV)', 'BRL' => 'Brazilian real (BRL)',
            'BSD' => 'Bahamian dollar (BSD)', 'BTN' => 'Bhutanese ngultrum (BTN)',
            'BWP' => 'Botswana pula (BWP)', 'BYN' => 'Belarusian ruble (BYN)',
            'BZD' => 'Belize dollar (BZD)', 'CAD' => 'Canadian dollar (CAD)',
            'CDF' => 'Congolese franc (CDF)', 'CHF' => 'Swiss franc (CHF)',
            'CHW' => 'WIR Franc (CHW)', 'CLF' => 'Unidad de Fomento (CLF)',
            'CLP' => 'Chilean peso (CLP)', 'CNY' => 'Chinese yuan (CNY)',
            'COP' => 'Colombian peso (COP)', 'COU' => 'Unidad de Valor Real (COU)',
            'CRC' => 'Costa Rican colon (CRC)', 'CUP' => 'Cuban peso (CUP)',
            'CVE' => 'Cape Verde escudo (CVE)', 'CZK' => 'Czech koruna (CZK)',
            'DJF' => 'Djiboutian franc (DJF)', 'DKK' => 'Danish krone (DKK)',
            'DOP' => 'Dominican peso (DOP)', 'DZD' => 'Algerian dinar (DZD)',
            'EGP' => 'Egyptian pound (EGP)', 'ERN' => 'Eritrean nakfa (ERN)',
            'ETB' => 'Ethiopian birr (ETB)', 'EUR' => 'Euro (EUR)',
            'FJD' => 'Fiji dollar (FJD)', 'FKP' => 'Falkland Islands pound (FKP)',
            'GBP' => 'Pound sterling (GBP)', 'GEL' => 'Georgian lari (GEL)',
            'GHS' => 'Ghanaian cedi (GHS)', 'GIP' => 'Gibraltar pound (GIP)',
            'GMD' => 'Gambian dalasi (GMD)', 'GNF' => 'Guinean franc (GNF)',
            'GTQ' => 'Guatemalan quetzal (GTQ)', 'GYD' => 'Guyanese dollar (GYD)',
            'HKD' => 'Hong Kong dollar (HKD)', 'HNL' => 'Honduran lempira (HNL)',
            'HRK' => 'Croatian kuna (HRK)', 'HTG' => 'Haitian gourde (HTG)',
            'HUF' => 'Hungarian forint (HUF)', 'IDR' => 'Indonesian rupiah (IDR)',
            'ILS' => 'Israeli new shekel (ILS)', 'INR' => 'Indian rupee (INR)',
            'IQD' => 'Iraqi dinar (IQD)', 'IRR' => 'Iranian rial (IRR)',
            'ISK' => 'Icelandic króna (ISK)', 'JMD' => 'Jamaican dollar (JMD)',
            'JOD' => 'Jordanian dinar (JOD)', 'JPY' => 'Japanese yen (JPY)',
            'KES' => 'Kenyan shilling (KES)', 'KGS' => 'Kyrgyzstani som (KGS)',
            'KHR' => 'Cambodian riel (KHR)', 'KMF' => 'Comoro franc (KMF)',
            'KPW' => 'North Korean won (KPW)', 'KRW' => 'South Korean won (KRW)',
            'KWD' => 'Kuwaiti dinar (KWD)', 'KYD' => 'Cayman Islands dollar (KYD)',
            'KZT' => 'Kazakhstani tenge (KZT)', 'LAK' => 'Lao kip (LAK)',
            'LBP' => 'Lebanese pound (LBP)', 'LKR' => 'Sri Lankan rupee (LKR)',
            'LRD' => 'Liberian dollar (LRD)', 'LSL' => 'Lesotho loti (LSL)',
            'LYD' => 'Libyan dinar (LYD)', 'MAD' => 'Moroccan dirham (MAD)',
            'MDL' => 'Moldovan leu (MDL)', 'MGA' => 'Malagasy ariary (MGA)',
            'MKD' => 'Macedonian denar (MKD)', 'MNT' => 'Mongolian tugrik (MNT)',
            'MOP' => 'Macanese pataca (MOP)', 'MUR' => 'Mauritian rupee (MUR)',
            'MVR' => 'Maldivian rufiyaa (MVR)', 'MWK' => 'Malawian kwacha (MWK)',
            'MXN' => 'Mexican peso (MXN)', 'MXV' => 'Mexican Unidad de Inversion (MXV)',
            'MYR' => 'Malaysian ringgit (MYR)', 'MZN' => 'Mozambican metical (MZN)',
            'NAD' => 'Namibian dollar (NAD)', 'NGN' => 'Nigerian naira (NGN)',
            'NIO' => 'Nicaraguan córdoba (NIO)', 'NOK' => 'Norwegian krone (NOK)',
            'NPR' => 'Nepalese rupee (NPR)', 'NZD' => 'New Zealand dollar (NZD)',
            'OMR' => 'Omani rial (OMR)', 'PAB' => 'Panamanian balboa (PAB)',
            'PEN' => 'Peruvian nuevo sol (PEN)', 'PGK' => 'Papua New Guinean kina (PGK)',
            'PHP' => 'Philippine peso (PHP)', 'PKR' => 'Pakistani rupee (PKR)',
            'PLN' => 'Polish złoty (PLN)', 'PYG' => 'Paraguayan guaraní (PYG)',
            'QAR' => 'Qatari riyal (QAR)', 'RON' => 'Romanian new leu (RON)',
            'RUB' => 'Russian rouble (RUB)', 'RWF' => 'Rwandan franc (RWF)',
            'SAR' => 'Saudi riyal (SAR)', 'SBD' => 'Solomon Islands dollar (SBD)',
            'SCR' => 'Seychelles rupee (SCR)', 'SDG' => 'Sudanese pound (SDG)',
            'SEK' => 'Swedish krona (SEK)', 'SGD' => 'Singapore dollar (SGD)',
            'SHP' => 'Saint Helena pound (SHP)', 'SLL' => 'Sierra Leonean leone (SLL)',
            'SOS' => 'Somali shilling (SOS)', 'SRD' => 'Surinamese dollar (SRD)',
            'SVC' => 'El Salvador colon (SVC)', 'SYP' => 'Syrian pound (SYP)',
            'SZL' => 'Swazi lilangeni (SZL)', 'THB' => 'Thai baht (THB)',
            'TJS' => 'Tajikistani somoni (TJS)', 'TND' => 'Tunisian dinar (TND)',
            'TOP' => 'Tongan paʻanga (TOP)', 'TRY' => 'Turkish lira (TRY)',
            'TTD' => 'Trinidad and Tobago dollar (TTD)', 'TWD' => 'New Taiwan dollar (TWD)',
            'TZS' => 'Tanzanian shilling (TZS)', 'UAH' => 'Ukrainian hryvnia (UAH)',
            'UGX' => 'Ugandan shilling (UGX)', 'USD' => 'United States dollar (USD)',
            'USN' => 'United States dollar (USN)', 'UYU' => 'Uruguayan peso (UYU)',
            'UZS' => 'Uzbekistan som (UZS)', 'VEF' => 'Venezuelan bolívar fuerte (VEF)',
            'VND' => 'Vietnamese dong (VND)', 'VUV' => 'Vanuatu vatu (VUV)',
            'WST' => 'Samoan tala (WST)', 'XAF' => 'CFA franc BEAC (XAF)',
            'XCD' => 'East Caribbean dollar (XCD)', 'XOF' => 'CFA franc BCEAO (XOF)',
            'XPF' => 'CFP franc (XPF)', 'YER' => 'Yemeni rial (YER)',
            'ZAR' => 'South African rand (ZAR)', 'ZWL' => 'Zimbabwe dollar (ZWL)'
        ];
        $currencyOptions = [];
        foreach ($currencies as $eachCurrencyCode => $eachCurrency) {
            $currencyOptions[] = [
                'value' => $eachCurrencyCode,
                'label' => [
                    'de-DE' => $eachCurrency,
                    'en-GB' => $eachCurrency,
                    Defaults::LANGUAGE_SYSTEM => $eachCurrency
                ]
            ];
        }
        return $currencyOptions;
    }

    /**
     * function to get Saferpay Shopware Pre Authorization Custom Fields
     *
     * @return array
     */
    protected function getSaferpayShopwarePreAuthorizationCustomFields(): array
    {
        return [
            'id' => '280e046389e64ba9a5f4dfc930fc5305',
            'name' => 'saferpay_shopware_pre_authorizations',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Pre-Authorizations',
                    'de-DE' => 'Vorabgenehmigungen',
                    Defaults::LANGUAGE_SYSTEM => 'Pre-Authorizations',
                ],
                'helpText' => [
                    'en-GB' => 'It extends the validity of an authorization to up to 30 days.',
                    'de-DE' => 'Es verlängert die Gültigkeit einer Autorisierung auf bis zu 30 Tage.',
                    Defaults::LANGUAGE_SYSTEM => 'It extends the validity of an authorization to up to 30 days.',
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => '1',
                        'label' => [
                            'de-DE' => 'Ja',
                            'en-GB' => 'Yes',
                            Defaults::LANGUAGE_SYSTEM => 'Yes',
                        ]
                    ],
                    [
                        'value' => '0',
                        'label' => [
                            'de-DE' => 'Nein',
                            'en-GB' => 'No',
                            Defaults::LANGUAGE_SYSTEM => 'No',
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * function to get SaferPay Shopware Capturing Custom Fields
     *
     * @return array
     */
    protected function getSaferPayShopwareCapturingCustomFields(): array
    {
        $autoCaptureText = 'Auto Capture';
        $manualCaptureText = 'Manual Capture';
        return [
            'id' => '3a1ec9fa2afa11eebe560242ac120002',
            'name' => 'saferpay_shopware_capturing',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Capturing',
                    'de-DE' => 'Capturing',
                    Defaults::LANGUAGE_SYSTEM => 'Capturing',
                ],
                'helpText' => [
                    'en-GB' => 'Should the amount be captured automatically after the order or
                    should the amount only be reserved and capture manually ? (Set Pre-Authorizations NO to enable Auto
                    Capture)',
                    'de-DE' => 'Soll der Betrag nach der Bestellung automatisch erfasst werden bzw. soll
                    Soll der Betrag nur manuell reserviert und erfasst werden? (Stellen Sie Vorautorisierungen auf NEIN ein, um „Automatisch“ zu aktivieren
                     Erfassen)',
                    Defaults::LANGUAGE_SYSTEM => 'Should the amount be captured automatically after the order or should
                    the amount only be reserved and capture manually ? (Set Pre-Authorizations NO to enable Auto
                     Capture)',
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => 'direct',
                        'label' => [
                            'de-DE' => $autoCaptureText,
                            'en-GB' => $autoCaptureText,
                            Defaults::LANGUAGE_SYSTEM => $autoCaptureText,
                        ]
                    ],
                    [
                        'value' => 'delayed',
                        'label' => [
                            'de-DE' => $manualCaptureText,
                            'en-GB' => $manualCaptureText,
                            Defaults::LANGUAGE_SYSTEM => $manualCaptureText,
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * function to get Saferpay Shopware Account Details Custom Fields
     *
     * @return array
     */
    protected function getSaferpayShopwareAccountDetailsCustomFields(): array
    {
        return [
            'id' => 'fd9788dbc4d544a6a7278faf5ce01547',
            'name' => 'saferpay_shopware_account_details',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Account Details',
                    'de-DE' => 'Kontodetails',
                    Defaults::LANGUAGE_SYSTEM => 'Account Details'
                ],
                'helpText' => [
                    'en-GB' => 'Bank details for pre-selection in checkout page',
                    'de-DE' => 'Bankverbindung zur Vorauswahl auf der Checkout-Seite',
                    Defaults::LANGUAGE_SYSTEM => 'Bank details for pre-selection in checkout page'
                ],
                'componentName' => 'sw-multi-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => $this->getIdealBankOptions()
            ]
        ];
    }

    /**
     * function to get Test Account Details Custom Fields
     *
     * @return array
     */
    protected function getTestAccountDetailsCustomFields(): array
    {
        $testBank1Text = 'Test Bank 1';
        $testBank2Text = 'Test Bank 2';
        return [
            'id' => '169e7eec9ace49c6a47f7d1be95ef546',
            'name' => 'saferpay_shopware_test_account_details',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Test Account Details',
                    'de-DE' => 'Testkontodetails',
                    Defaults::LANGUAGE_SYSTEM => 'Test Account Details'
                ],
                'helpText' => [
                    'en-GB' => 'Test Bank details for pre-selection in checkout page',
                    'de-DE' => 'Testen Sie die Bankdaten zur Vorauswahl auf der Checkout-Seite',
                    Defaults::LANGUAGE_SYSTEM => 'Test Bank details for pre-selection in checkout page'
                ],
                'componentName' => 'sw-multi-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => '0091',
                        'label' => [
                            'en-GB' => $testBank1Text,
                            'de-DE' => $testBank1Text,
                            Defaults::LANGUAGE_SYSTEM => $testBank1Text
                        ]
                    ],
                    [
                        'value' => '0092',
                        'label' => [
                            'en-GB' => $testBank2Text,
                            'de-DE' => $testBank2Text,
                            Defaults::LANGUAGE_SYSTEM => $testBank2Text
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * Get the list of banks for ideal payment method
     */
    private function getIdealBankOptions()
    {
        $idealPaymentBanks = [
            'ABNANL2A' => 'ABN AMRO',
            'ASNBNL21' => 'ASN Bank',
            'BUNQNL2A' => 'bunq',
            'INGBNL2A' => 'ING',
            'KNABNL2H' => 'Knab',
            'RABONL2U' => 'Rabobank',
            'RBRBNL21' => 'RegioBank',
            'REVOLT21' => 'Revolut',
            'SNSBNL2A' => 'SNS',
            'TRIONL2U' => 'Triodos Bank',
            'FVLBNL22' => 'Van Lanschot',
            'BITSNL2A' => 'Yoursafe B.V.'
        ];
        $options = array();
        foreach ($idealPaymentBanks as $key => $value) {
            $options[] = [
                'value' => $key,
                'label' => [
                    'de-DE' => $value,
                    'en-GB' => $value,
                    Defaults::LANGUAGE_SYSTEM => $value
                ]
            ];
        }
        return $options;
    }

    /**
     * function to get Shopware Authorization Method Custom Fields
     *
     * @return array
     */
    protected function getShopwareAuthorizationMethodCustomFields(): array
    {

        $paymentPageText = 'Payment Page';
        $widgetText = 'Widget Authorization';
        return [
            'id' => '3ca6ce9529a4453d8947680a20e9569b',
            'name' => 'saferpay_shopware_authorization_method',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Authorization Method',
                    'de-DE' => 'Autorisierungsmethode',
                    Defaults::LANGUAGE_SYSTEM => 'Authorization Method'
                ],
                'helpText' => [
                    'en-GB' => 'Select the authorization method to use for processing this payment method.
                     The Widget Authorization method is only for holders of a business licence on the live system.',
                    'de-DE' => 'Wählen Sie die Autorisierungsmethode aus, die für die Verarbeitung dieser
                    Zahlungsmethode verwendet werden soll. Die Widget-Autorisierungsmethode ist nur für Inhaber
                     einer Geschäftslizenz auf dem Live-System verfügbar.',
                    Defaults::LANGUAGE_SYSTEM => 'Select the authorization method to use for processing this payment
                     method. The Widget Authorization method is only for holders of a business licence on the live
                    system.'
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => 'paymentPage',
                        'label' => [
                            'en-GB' => $paymentPageText,
                            'de-DE' => $paymentPageText,
                            Defaults::LANGUAGE_SYSTEM => $paymentPageText
                        ]
                    ],
                    [
                        'value' => 'widgetAuthorization',
                        'label' => [
                            'en-GB' => $widgetText,
                            'de-DE' => $widgetText,
                            Defaults::LANGUAGE_SYSTEM => $widgetText
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * function to get Shopware Authorization Method Custom Fields
     *
     * @return array
     */
    protected function getSaveCardCustomField(): array
    {
        return [
            'id' => 'e636df184c5842fea55910ece423ab61',
            'name' => 'saferpay_shopware_save_card',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Save Card',
                    'de-DE' => 'Karte speichern',
                    Defaults::LANGUAGE_SYSTEM => 'Save Card'
                ],
                'helpText' => [
                    'en-GB' => 'Choose whether Save Card option should be allowed for this payment method.
                     The sensitive data is stored by Saferpay.',
                    'de-DE' => 'Wählen Sie aus, ob die Option Karte speichern für diese Zahlungsmethode zulässig
                     sein soll. Die sensiblen Daten werden von Saferpay gespeichert.',
                    Defaults::LANGUAGE_SYSTEM => 'Choose whether Save Card option should be allowed for this
                     payment method. The sensitive data is stored by Saferpay.'
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => 'active',
                        'label' => [
                            'en-GB' => 'Active',
                            'de-DE' => 'Aktiv',
                            Defaults::LANGUAGE_SYSTEM => 'Active'
                        ]
                    ],
                    [
                        'value' => 'inactive',
                        'label' => [
                            'en-GB' => 'Inactive',
                            'de-DE' => 'Inaktiv',
                            Defaults::LANGUAGE_SYSTEM => 'Inactive'
                        ]
                    ]
                ]
            ]
        ];
    }

    protected function getSaferpayFieldAccessToken()
    {
        return [
            'id' => 'fc314b00c4ea11eea5060242ac120002',
            'name' => 'saferpay_shopware_field_access_token',
            'type' => CustomFieldTypes::TEXT,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'text',
                'label' => [
                    'en-GB' => self::SAFERPAY_FIELDS_ACCESS_TOKEN,
                    'de-DE' => self::SAFERPAY_FIELDS_ACCESS_TOKEN,
                    Defaults::LANGUAGE_SYSTEM => self::SAFERPAY_FIELDS_ACCESS_TOKEN
                ],
                'helpText' => [
                    'en-GB' => 'You can create this from Saferpay Backoffice. After login navigate to Settings >
                    Saferpay Fields Access Tokens.',
                    'de-DE' => 'Sie können dies im Saferpay Backoffice erstellen. Navigieren Sie nach der Anmeldung zu
                    Einstellungen> Saferpay Fields Access Tokens.',
                    Defaults::LANGUAGE_SYSTEM => 'You can create this from Saferpay Backoffice.
                    After login navigate to Settings > Saferpay Fields Access Tokens.'
                ],
                'componentName' => 'sw-text-field',
                'customFieldType' => 'text',
                'customFieldPosition' => 60
            ]
        ];
    }

    protected function getSaferpayFieldJsLibraryUrl()
    {
        return [
            'id' => '0d2ded1ec4eb11eea5060242ac120002',
            'name' => 'saferpay_shopware_field_js_library_url',
            'type' => CustomFieldTypes::TEXT,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'text',
                'label' => [
                    'en-GB' => self::SAFERPAY_FIELDS_JS_LIBRARY_URL,
                    'de-DE' => self::SAFERPAY_FIELDS_JS_LIBRARY_URL,
                    Defaults::LANGUAGE_SYSTEM => self::SAFERPAY_FIELDS_JS_LIBRARY_URL
                ],
                'helpText' => [
                    'en-GB' => 'You can define here a CSS URL. The CSS can only be used, when it is hosted on a
                    server which has SSL enabled.',
                    'de-DE' => 'Hier können Sie eine CSS-URL definieren. Das CSS kann nur verwendet werden,
                     wenn es auf einem Server gehostet wird, auf dem SSL aktiviert ist.',
                    Defaults::LANGUAGE_SYSTEM => 'You can define here a CSS URL. The CSS can only be used, when
                     it is hosted on a server which has SSL enabled.'
                ],
                'componentName' => 'sw-text-field',
                'customFieldType' => 'text',
                'customFieldPosition' => 80,
            ]
        ];
    }

    protected function getSaferpayFieldCssUrl()
    {
        return [
            'id' => '1222d4b48a0040978085994ea02174de',
            'name' => 'saferpay_shopware_field_css_url',
            'type' => CustomFieldTypes::TEXT,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'text',
                'label' => [
                    'en-GB' => self::SAFERPAY_FIELDS_CSS_URL,
                    'de-DE' => self::SAFERPAY_FIELDS_CSS_URL,
                    Defaults::LANGUAGE_SYSTEM => self::SAFERPAY_FIELDS_CSS_URL
                ],
                'helpText' => [
                    'en-GB' => 'Css URL You can define here a CSS URL. The CSS can only be used,
                     when it is hosted on a server which has SSL enabled.',
                    'de-DE' => 'CSS-URL Hier können Sie eine CSS-URL definieren. Das CSS kann nur verwendet werden,
                    wenn es auf einem Server gehostet wird, auf dem SSL aktiviert ist.',
                    Defaults::LANGUAGE_SYSTEM => 'Css URL You can define here a CSS URL. The CSS can only be used,
                     when it is hosted on a server which has SSL enabled..'
                ],
                'componentName' => 'sw-text-field',
                'customFieldType' => 'text',
                'customFieldPosition' => 80,
            ]
        ];
    }

    protected function getSaferpayFieldApiUrl()
    {
        return [
            'id' => '194f678ac4eb11eea5060242ac120002',
            'name' => 'saferpay_shopware_field_api_url',
            'type' => CustomFieldTypes::TEXT,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'text',
                'label' => [
                    'en-GB' => self::SAFERPAY_FIELDS_APIURL_TEXT,
                    'de-DE' => self::SAFERPAY_FIELDS_APIURL_TEXT,
                    Defaults::LANGUAGE_SYSTEM => self::SAFERPAY_FIELDS_APIURL_TEXT
                ],
                'helpText' => [
                    'en-GB' => 'Contains the API-Url, to define, where to post the data and initialize the Saferpay
                    Fields. Test Environment url: "https://test.saferpay.com/Fields/[YOUR CUSTOMERID]"
                    Live Environment url: "https://www.saferpay.com/Fields/[YOUR CUSTOMERID]"',
                    'de-DE' => 'Enthält die API-URL, um zu definieren, wo die Daten gepostet werden sollen und um Saferpay zu initialisieren
                    Felder. URL der Testumgebung: "https://test.saferpay.com/Fields/[YOUR CUSTOMERID]"
                    Live Environment url: "https://www.saferpay.com/Fields/[YOUR CUSTOMERID]"',
                    Defaults::LANGUAGE_SYSTEM => 'Contains the API-Url, to define, where to post the data
                    and initialize the Saferpay Fields. Test Environment url:
                    "https://test.saferpay.com/Fields/[YOUR CUSTOMERID] Live Environment url:
                    "https://www.saferpay.com/Fields/[YOUR CUSTOMERID]"'
                ],
                'componentName' => 'sw-text-field',
                'customFieldType' => 'text',
                'customFieldPosition' => 100,
            ]
        ];
    }

    protected function getCreditCardTheme()
    {
        return [
            'id' => 'b915ec5cd3e6469789b9256369afe891',
            'name' => 'saferpay_credit_card_theme',
            'type' => CustomFieldTypes::SELECT,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'text',
                'label' => [
                    'en-GB' => self::SAFERPAY_CREDIT_CARD_THEME,
                    'de-DE' => self::SAFERPAY_CREDIT_CARD_THEME,
                    Defaults::LANGUAGE_SYSTEM => self::SAFERPAY_CREDIT_CARD_THEME
                ],
                'helpText' => [
                    'en-GB' => 'The backend of Saferpay allows the definition of credit card configurations.
                     This field contains the name of such a configuraiton name.
                     The configuration allows the detailed control over the layout of the credit card field.',
                    'de-DE' => 'Das Backend von Saferpay ermöglicht die Definition von Kreditkartenkonfigurationen.
                    Dieses Feld enthält den Namen eines solchen Konfigurationsnamens.
                    Die Konfiguration ermöglicht die detaillierte Kontrolle über das Layout des Kreditkartenfeldes',
                    Defaults::LANGUAGE_SYSTEM => 'You can create this from Saferpay Backoffice.
                    After login navigate to Settings > Saferpay Fields Access Tokens.'
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => 'sample_1',
                        'label' => [
                            'en-GB' => 'Sample 1',
                            'de-DE' => 'Probe 1',
                            Defaults::LANGUAGE_SYSTEM => 'Sample 1'
                        ]
                    ],
                    [
                        'value' => 'sample_2',
                        'label' => [
                            'en-GB' => 'Sample 2',
                            'de-DE' => 'Probe 2',
                            Defaults::LANGUAGE_SYSTEM => 'Sample 2'
                        ]
                    ],
                    [
                        'value' => 'sample_3',
                        'label' => [
                            'en-GB' => 'Sample 3',
                            'de-DE' => 'Probe 3',
                            Defaults::LANGUAGE_SYSTEM => 'Sample 3'
                        ]
                    ],
                    [
                        'value' => 'sample_4',
                        'label' => [
                            'en-GB' => 'Sample 4',
                            'de-DE' => 'Probe 4',
                            Defaults::LANGUAGE_SYSTEM => 'Sample 4'
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * Check if authorization method custom field is to be added to a payment method
     */
    protected function checkIfAuthorizationMethodPaymentMethod($paymentMethod): bool
    {
        if (in_array($paymentMethod, [
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID
        ])) {
            return true;
        }
        return false;
    }

    /**
     * Check if payment method has pre authorization enabled
     */
    protected function checkIfPreAuthPaymentMethod($paymentMethod): bool
    {
        if (in_array($paymentMethod, [
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID
        ])) {
            return true;
        }
        return false;
    }

    /**
     * Check if capturing enabled payment method
     */
    protected function checkIfCapturingPaymentMethod($paymentMethod): bool
    {
        if (in_array($paymentMethod, [
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
            SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID,
            SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID
        ])) {
            return true;
        }
        return false;
    }

    /**
     * Check if the payment method has save card option
     */
    protected function checkIfSaveCardPaymentMethod($paymentMethod): bool
    {
        if (in_array($paymentMethod, [
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID,
        ])) {
            return true;
        }
        return false;
    }

    /**
     * Config to display the card holder name
     */
    protected function displayCardHolderNameConfig(): array
    {
        return [
            'id' => '3ca6ce9529a4453d8947680a20e9569c',
            'name' => 'saferpay_shopware_holder_name_field',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Card Holder Name Display',
                    'de-DE' => 'Name des Karteninhabers Anzeige',
                    Defaults::LANGUAGE_SYSTEM => 'Card Holder Name Display'
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => 'yes',
                        'label' => [
                            'en-GB' => "Yes",
                            'de-DE' => "ja",
                            Defaults::LANGUAGE_SYSTEM => "yes"
                        ]
                    ],
                    [
                        'value' => 'No',
                        'label' => [
                            'en-GB' => "No",
                            'de-DE' => "Non",
                            Defaults::LANGUAGE_SYSTEM => "Yes"
                        ]
                    ]
                ]
            ]
        ];
    }
}
