<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Migration;

use Doctrine\DBAL\Connection;
use Shopware\Core\Framework\Migration\MigrationStep;

class Migration1689835564SaferpayPaymentTransactionAlterForeignKey extends MigrationStep
{
    public function getCreationTimestamp(): int
    {
        return 1689835564;
    }

    public function update(Connection $connection): void
    {
        // implement update
        $sql = <<<SQL
-- Drop the foreign keys if they exist
SET @var1 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction'
            AND CONSTRAINT_NAME = 'saferpay_payment_transaction_ibfk_1' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) > 0,
    'ALTER TABLE `saferpay_payment_transaction` DROP FOREIGN KEY `saferpay_payment_transaction_ibfk_1`',
    'SELECT 1'
);

PREPARE stmt1 FROM @var1;
EXECUTE stmt1;
DEALLOCATE PREPARE stmt1;

SET @var2 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'saferpay_payment_transaction_ibfk_2' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) > 0,
    'ALTER TABLE `saferpay_payment_transaction` DROP FOREIGN KEY `saferpay_payment_transaction_ibfk_2`',
    'SELECT 1'
);

PREPARE stmt2 FROM @var2;
EXECUTE stmt2;
DEALLOCATE PREPARE stmt2;

SET @var3 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'saferpay_payment_transaction_ibfk_3' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) > 0,
    'ALTER TABLE `saferpay_payment_transaction` DROP FOREIGN KEY `saferpay_payment_transaction_ibfk_3`',
    'SELECT 1'
);

PREPARE stmt3 FROM @var3;
EXECUTE stmt3;
DEALLOCATE PREPARE stmt3;

SET @var4 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'saferpay_payment_transaction_ibfk_4' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) > 0,
    'ALTER TABLE `saferpay_payment_transaction` DROP FOREIGN KEY `saferpay_payment_transaction_ibfk_4`',
    'SELECT 1'
);

PREPARE stmt4 FROM @var4;
EXECUTE stmt4;
DEALLOCATE PREPARE stmt4;

-- Insert new foreign key if not exists
SET @var5 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'fk.saferpay_payment_transaction.payment_id' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) = 0,
    'ALTER TABLE `saferpay_payment_transaction` ADD CONSTRAINT `fk.saferpay_payment_transaction.payment_id`
    FOREIGN KEY (`payment_id`) REFERENCES `payment_method`(`id`) ON DELETE CASCADE ON UPDATE CASCADE',
    'SELECT 1'
);

PREPARE stmt5 FROM @var5;
EXECUTE stmt5;
DEALLOCATE PREPARE stmt5;

SET @var6 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'fk.saferpay_payment_transaction.order_id' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) = 0,
    'ALTER TABLE `saferpay_payment_transaction` ADD CONSTRAINT `fk.saferpay_payment_transaction.order_id`
    FOREIGN KEY (`order_id`) REFERENCES `order`(`id`) ON DELETE CASCADE ON UPDATE CASCADE',
    'SELECT 1'
);

PREPARE stmt6 FROM @var6;
EXECUTE stmt6;
DEALLOCATE PREPARE stmt6;

SET @var7 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'fk.saferpay_payment_transaction.customer_id' AND CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) = 0,
    'ALTER TABLE `saferpay_payment_transaction` ADD CONSTRAINT `fk.saferpay_payment_transaction.customer_id`
    FOREIGN KEY (`customer_id`) REFERENCES `customer`(`id`) ON DELETE CASCADE ON UPDATE CASCADE',
    'SELECT 1'
);

PREPARE stmt7 FROM @var7;
EXECUTE stmt7;
DEALLOCATE PREPARE stmt7;

SET @var8 = IF(
    (
        SELECT COUNT(*)
        FROM information_schema.TABLE_CONSTRAINTS
        WHERE
            CONSTRAINT_SCHEMA = DATABASE() AND
            TABLE_NAME = 'saferpay_payment_transaction' AND
            CONSTRAINT_NAME = 'fk.saferpay_payment_transaction.order_transaction_id' AND
            CONSTRAINT_TYPE = 'FOREIGN KEY'
    ) = 0
    AND (
        SELECT COUNT(*)
        FROM information_schema.COLUMNS
        WHERE
            TABLE_SCHEMA = DATABASE() AND
            TABLE_NAME = 'saferpay_payment_transaction' AND
            COLUMN_NAME = 'order_transaction_id'
    ) > 0,
    'ALTER TABLE `saferpay_payment_transaction` ADD CONSTRAINT `fk.saferpay_payment_transaction.order_transaction_id`
    FOREIGN KEY (`order_transaction_id`) REFERENCES `order_transaction`(`id`) ON DELETE CASCADE ON UPDATE CASCADE',
    'SELECT 1'
);


PREPARE stmt8 FROM @var8;
EXECUTE stmt8;
DEALLOCATE PREPARE stmt8;

SQL;

$connection->executeStatement($sql);

    }

    public function updateDestructive(Connection $connection): void
    {
        // implement update destructive
    }
}
