<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Migration\Test;

use Doctrine\DBAL\Connection;
use PHPUnit\Framework\TestCase;
use Saferpay\Shopware\Migration\Migration1689659056SaferpayPaymentTransaction;
use Saferpay\Shopware\Migration\Migration1689685221SaferpayPaymentTransactionAlter;
use Saferpay\Shopware\Migration\Migration1689835564SaferpayPaymentTransactionAlterForeignKey;
use Shopware\Core\Framework\Test\TestCaseBase\KernelTestBehaviour;

class Migration1689659056SaferpayPaymentTransactionTest extends TestCase
{
    use KernelTestBehaviour;

    public function testNoChanges(): void
    {
        $showCreateTableSql = "SHOW CREATE TABLE `saferpay_payment_transaction`";
        /** @var Connection $conn */
        $conn = $this->getContainer()->get(Connection::class);
        $expectedSchema = $conn->fetchAllAssociative($showCreateTableSql);

        $migration = new Migration1689659056SaferpayPaymentTransaction();
        $migration->update($conn);

        $migration2 = new Migration1689685221SaferpayPaymentTransactionAlter();
        $migration2->update($conn);

        $migration3 = new Migration1689835564SaferpayPaymentTransactionAlterForeignKey();
        $migration3->update($conn);

        $actualSchema = $conn->fetchAllAssociative($showCreateTableSql);
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');

        $migration->updateDestructive($conn);
        $migration2->updateDestructive($conn);
        $migration3->updateDestructive($conn);

        $actualSchema = $conn->fetchAllAssociative($showCreateTableSql);
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');
    }

    public function testNoTable(): void
    {
        /** @var Connection $conn */
        $conn = $this->getContainer()->get(Connection::class);
        $conn->executeStatement('DROP TABLE `saferpay_payment_transaction`');

        $migration = new Migration1689659056SaferpayPaymentTransaction();
        $migration->update($conn);

        $migration2 = new Migration1689685221SaferpayPaymentTransactionAlter();
        $migration2->update($conn);

        $migration3 = new Migration1689835564SaferpayPaymentTransactionAlterForeignKey();
        $migration3->update($conn);

        $exists = $conn->fetchFirstColumn('SELECT COUNT(*) FROM `saferpay_payment_transaction`') !== false;

        static::assertTrue($exists);
    }
}
