<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Migration\Test;

use Doctrine\DBAL\Connection;
use PHPUnit\Framework\TestCase;
use Saferpay\Shopware\Migration\Migration1689671000SaferpayTransactionHistory;
use Saferpay\Shopware\Migration\Migration1689844546SaferpayTransactionHistoryAlterForeignKey;
use Saferpay\Shopware\Migration\Migration1690960809SaferpayTransactionHistoryForeignKeyEdit;
use Saferpay\Shopware\Migration\Migration1691569958SaferpayTransactionHistoryUpdate;
use Shopware\Core\Framework\Test\TestCaseBase\KernelTestBehaviour;

class Migration1689671000SaferpayTransactionHistoryTest extends TestCase
{
    use KernelTestBehaviour;

    public function testNoChanges(): void
    {
        $showCreateTableSql = "SHOW CREATE TABLE `saferpay_transaction_history`";
        /** @var Connection $conn */
        $conn = $this->getContainer()->get(Connection::class);
        $expectedSchema = $conn->fetchAllAssociative($showCreateTableSql);

        $migration = new Migration1689671000SaferpayTransactionHistory();
        $migration->update($conn);

        $migration2 = new Migration1689844546SaferpayTransactionHistoryAlterForeignKey();
        $migration2->update($conn);
        $migration3 = new Migration1690960809SaferpayTransactionHistoryForeignKeyEdit();
        $migration3->update($conn);
        $migration4 = new Migration1691569958SaferpayTransactionHistoryUpdate();
        $migration4->update($conn);

        $actualSchema = $conn->fetchAllAssociative($showCreateTableSql);
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');

        $migration->updateDestructive($conn);
        $migration2->updateDestructive($conn);
        $migration3->updateDestructive($conn);
        $migration4->updateDestructive($conn);

        $actualSchema = $conn->fetchAllAssociative($showCreateTableSql);
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');
    }

    public function testNoTable(): void
    {
        /** @var Connection $conn */
        $conn = $this->getContainer()->get(Connection::class);
        $conn->executeStatement('DROP TABLE `saferpay_transaction_history`');

        $migration = new Migration1689671000SaferpayTransactionHistory();
        $migration->update($conn);
        $migration2 = new Migration1689844546SaferpayTransactionHistoryAlterForeignKey();
        $migration2->update($conn);
        $migration3 = new Migration1690960809SaferpayTransactionHistoryForeignKeyEdit();
        $migration3->update($conn);
        $migration4 = new Migration1691569958SaferpayTransactionHistoryUpdate();
        $migration4->update($conn);

        $exists = $conn->fetchFirstColumn('SELECT COUNT(*) FROM `saferpay_transaction_history`') !== false;

        static::assertTrue($exists);
    }
}
