<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Migration\Test;

use Doctrine\DBAL\Connection;
use PHPUnit\Framework\TestCase;
use Saferpay\Shopware\Migration\Migration1706684955SaferpayCustomerSecureTransaction;
use Shopware\Core\Framework\Test\TestCaseBase\KernelTestBehaviour;

class Migration1706684955SaferpayCustomerSecureTransactionTest extends TestCase
{
    use KernelTestBehaviour;

    public function testNoChanges(): void
    {
        $showCreateTableSql = "SHOW CREATE TABLE `saferpay_customer_secure_transaction`";
        /** @var Connection $conn */
        $conn = $this->getContainer()->get(Connection::class);
        $expectedSchema = $conn->fetchAllAssociative($showCreateTableSql);

        $migration = new Migration1706684955SaferpayCustomerSecureTransaction();

        $migration->update($conn);
        $actualSchema = $conn->fetchAllAssociative($showCreateTableSql);
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');

        $migration->updateDestructive($conn);
        $actualSchema = $conn->fetchAllAssociative($showCreateTableSql);
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');
    }

    public function testNoTable(): void
    {
        /** @var Connection $conn */
        $conn = $this->getContainer()->get(Connection::class);
        $conn->executeStatement('DROP TABLE `saferpay_customer_secure_transaction`');

        $migration = new Migration1706684955SaferpayCustomerSecureTransaction();
        $migration->update($conn);
        $exists = $conn->fetchFirstColumn('SELECT COUNT(*) FROM `saferpay_customer_secure_transaction`') !== false;

        static::assertTrue($exists);
    }
}
