import template from './sw-settings-payment-detail.html.twig';

const { Criteria } = Shopware.Data;

Shopware.Component.override('sw-settings-payment-detail', {
    template,

    data() {
        return {
            showNewAccountDetailCreationMessage: false,
            currentPaymentMethod: null,
            paymentTechnicalName: null
        };
    },

    inject: [
        'repositoryFactory',
    ],

    computed: {
        paymentRepository() {
            return this.repositoryFactory.create('payment_method');
        },
    },

    created() {
        this.checkCustomFieldInfo();
        this.loadAuthorizationMethodListener();
    },

    methods: {
        async checkCustomFieldsForPaymentMethod() {
            await this.$nextTick();

            const paymentId = this.$route.params.id;

            try {

                const criteria = new Criteria();
                criteria.addFilter(Criteria.equals('id', paymentId));
                this.paymentRepository.search(
                    criteria,
                    Shopware.Context.api
                ).then((result) => {
                    this.currentPaymentMethod = result;
                    this.paymentTechnicalName = this.currentPaymentMethod[0].technicalName;
                    if ((this.paymentTechnicalName != "SPSW - CREDIT_CARD") && (typeof document.getElementsByClassName('sw-custom-field-set-renderer-tab-content__saferpay_shopware_payment_method')[0] !== 'undefined')) {
                        document.getElementsByClassName('sw-custom-field-set-renderer-tab-content__saferpay_shopware_payment_method')[0].style = "pointer-events:none;";
                    }
                    if ((this.paymentTechnicalName == "SPSW - CREDIT_CARD") && (typeof document.getElementsByClassName('sw-inherit-wrapper sw-form-field-renderer-field__saferpay_shopware_wallets')[0] !== 'undefined')) {
                        document.getElementsByClassName('sw-inherit-wrapper sw-form-field-renderer-field__saferpay_shopware_wallets')[0].style = "pointer-events:none;";
                    }

                    const paymentMethodsOfSaferpay = [
                        'SPSW - ALIPAY', 'SPSW - AMEX', 'SPSW - BANCONTACT', 'SPSW - DINERS',
                        'SPSW - DIRECTDEBIT', 'SPSW - EPRZELEWY', 'SPSW - EPS', 'SPSW - IDEAL', 'SPSW - JCB', 'SPSW - KLARNA',
                        'SPSW - MAESTRO', 'SPSW - MASTERCARD', 'SPSW - MYONE', 'SPSW - PAYPAL', 'SPSW - PAYDIREKT', 'SPSW - POSTCARD',
                        'SPSW - POSTFINANCE', 'SPSW - TWINT', 'SPSW - UNIONPAY', 'SPSW - VISA', 'SPSW - WLCRYPTOPAYMENTS', 'SPSW - CREDIT_CARD',
                        'SPSW - APPLEPAY', 'SPSW - GOOGLEPAY', 'SPSW - SOFORT'
                    ];

                    const authorizationMethodArray = [
                        'SPSW - AMEX', 'SPSW - BANCONTACT', 'SPSW - CREDIT_CARD', 'SPSW - DINERS', 'SPSW - JCB', 'SPSW - MAESTRO',
                        'SPSW - MASTERCARD', 'SPSW - MYONE', 'SPSW - VISA'
                    ];

                    const onlyPaymentPageAuthorizationPayment = [
                        'SPSW - ALIPAY', 'SPSW - DIRECTDEBIT','SPSW - WLCRYPTOPAYMENTS','SPSW - SOFORT','SPSW - POSTCARD',
                        'SPSW - GOOGLEPAY','SPSW - PAYPAL','SPSW - APPLEPAY','SPSW - KLARNA','SPSW - PAYDIREKT','SPSW - POSTFINANCE',
                        'SPSW - EPS','SPSW - UNIONPAY','SPSW - TWINT','SPSW - IDEAL','SPSW - EPRZELEWY'
                    ];
  
                    const capturePaymentsArray = [
                        'SPSW - AMEX', 'SPSW - APPLEPAY', 'SPSW - CREDIT_CARD', 'SPSW - DINERS', 'SPSW - JCB', 'SPSW - MAESTRO', 'SPSW - MASTERCARD', 'SPSW - MYONE',
                        'SPSW - PAYPAL', 'SPSW - POSTCARD', 'SPSW - POSTFINANCE', 'SPSW - TWINT', 'SPSW - VISA',
                        'SPSW - KLARNA', 'SPSW - GOOGLEPAY', 'SPSW - DIRECTDEBIT'
                    ];
                    
                    const onlyAutoCapturePaymentMethods =[
                        'SPSW - ALIPAY', 'SPSW - BANCONTACT', 'SPSW - EPRZELEWY', 'SPSW - EPS', 'SPSW - IDEAL',
                        'SPSW - WLCRYPTOPAYMENTS', 'SPSW - UNIONPAY','SPSW - SOFORT','SPSW - PAYDIREKT'
                    ];
                    
                    const customFieldSetNames = [
                        'saferpay_shopware_payment_method', 'saferpay_shopware_payment_country_currency',
                        'saferpay_shopware_payment_order_total', 'saferpay_shopware_payment_customer',
                        'saferpay_shopware_payment_specific'
                    ];

                    const preAuthPaymentsArray = ['SPSW - AMEX', 'SPSW - DINERS', 'SPSW - JCB', 'SPSW - MAESTRO', 'SPSW - MASTERCARD', 'SPSW - VISA','SPSW - CREDIT_CARD'];

                    const saveCardArray = [
                        'SPSW - AMEX', 'SPSW - BANCONTACT', 'SPSW - CREDIT_CARD', 'SPSW - DINERS', 'SPSW - JCB', 'SPSW - MAESTRO', 'SPSW - MASTERCARD', 'SPSW - VISA','SPSW - TWINT'
                    ];
                    const cardHolderName = ['SPSW - ALIPAY', 'SPSW - WLCRYPTOPAYMENTS','SPSW - EPS','SPSW - TWINT','SPSW - EPRZELEWY','SPSW - IDEAL','SPSW - DIRECTDEBIT',
                        'SPSW - KLARNA','SPSW - PAYDIREKT','SPSW - PAYPAL','SPSW - SOFORT','SPSW - POSTFINANCE','SPSW - POSTCARD','SPSW - CREDIT_CARD'];

                    const customFieldSetRendererComponent = this.$refs.customFieldSetRenderer;

                    if (customFieldSetRendererComponent) {
                        const customFieldSetArray = customFieldSetRendererComponent.sets;
                        for (let i = customFieldSetArray.length - 1; i >= 0; i--) {
                            const customFieldSet = customFieldSetArray[i];
                            if (!paymentMethodsOfSaferpay.includes(this.paymentTechnicalName) && customFieldSetNames.includes(customFieldSet.name)) {
                                const customFieldSetPos = customFieldSetArray.findIndex(customFieldSetCheck => customFieldSetCheck.name === customFieldSet.name);
                                customFieldSetArray.splice(customFieldSetPos, 1);
                            }
                            if (customFieldSet.name === 'saferpay_shopware_payment_method' && this.paymentTechnicalName === 'SPSW - CREDIT_CARD') {
                                const paymentMethodCustomFields = customFieldSet.customFields;
                                for (let k = paymentMethodCustomFields.length - 1; k >= 0; k--) {
                                    const paymentCustomField = paymentMethodCustomFields[k];
                                    if (paymentCustomField.name === 'saferpay_shopware_payment_means') {
                                        let allowedPaymentTypes = ['AMEX', 'BANCONTACT', 'DINERS', 'JCB', 'MAESTRO', 'MASTERCARD', 'VISA'];
                                        let config = paymentCustomField.config;
                                        let options = config.options;
                                        options = options.filter(option => allowedPaymentTypes.includes(option.value));
                                        config.options = options;
                                    }
                                }
                            }
                            if (customFieldSet.name === 'saferpay_shopware_payment_specific') {
                                const specificCustomFields = customFieldSet.customFields;
                                for (let j = specificCustomFields.length - 1; j >= 0; j--) {
                                    const customField = specificCustomFields[j];
                                    if (customField.name === 'saferpay_shopware_authorization_method' && this.paymentTechnicalName === 'SPSW - CREDIT_CARD') {
                                        let config = customField.config;
                                        let options = config.options;
                                        let optionToRemove = "paymentPage";
                                        options = options.filter(option => option.value !== optionToRemove);
                                        config.options = options;
                                    }
                                    if (customField.name === 'saferpay_shopware_capturing' && capturePaymentsArray.includes(this.paymentTechnicalName) == false) {
                                        if(onlyAutoCapturePaymentMethods.includes(this.paymentTechnicalName) == true){
                                            let config = customField.config;
                                            let options = config.options;
                                            let optionToRemove = "delayed";
                                            options = options.filter(option => option.value !== optionToRemove);
                                            config.options = options;
                                        }else{
                                            this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                        }
                                    }
                                    if (customField.name === 'saferpay_shopware_pre_authorizations' && preAuthPaymentsArray.includes(this.paymentTechnicalName) == false) {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_account_details' && this.paymentTechnicalName !== 'SPSW - IDEAL') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_test_account_details' && this.paymentTechnicalName !== 'SPSW - IDEAL') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_field_access_token' && this.paymentTechnicalName !== 'SPSW - CREDIT_CARD') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_field_js_library_url' && this.paymentTechnicalName !== 'SPSW - CREDIT_CARD') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_field_api_url' && this.paymentTechnicalName !== 'SPSW - CREDIT_CARD') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_credit_card_theme' && this.paymentTechnicalName !== 'SPSW - CREDIT_CARD') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_field_css_url' && this.paymentTechnicalName !== 'SPSW - CREDIT_CARD') {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_authorization_method' && authorizationMethodArray.includes(this.paymentTechnicalName) == false) {
                                        if(onlyPaymentPageAuthorizationPayment.includes(this.paymentTechnicalName) == true){
                                            let config = customField.config;
                                            let options = config.options;
                                            let optionToRemove = "widgetAuthorization";
                                            options = options.filter(option => option.value !== optionToRemove);
                                            config.options = options;
                                        }else {
                                            this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                        }
                                    }
                                    if (customField.name === 'saferpay_shopware_save_card' && saveCardArray.includes(this.paymentTechnicalName) == false) {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                    if (customField.name === 'saferpay_shopware_holder_name_field' && cardHolderName.includes(this.paymentTechnicalName)) {
                                        this.removeCustomFieldFromSet(customFieldSet.name, customField.name);
                                    }
                                }
                            }
                        }
                    }
                });
            } catch (error) {
                console.error('Error fetching payment details:', error);
            }
        },

        removeCustomFieldFromSet(customFieldSetName, customFieldName) {
            let customFieldSetRendererComponent = this.$refs.customFieldSetRenderer;

            if (customFieldSetRendererComponent) {

                const customFieldSetArray = customFieldSetRendererComponent.sets;
                const targetCustomFieldSetPos = customFieldSetArray.findIndex(customFieldSet => customFieldSet.name === customFieldSetName);

                if (targetCustomFieldSetPos !== -1) {
                    const targetCustomFieldIndex = customFieldSetArray[targetCustomFieldSetPos].customFields.findIndex(customField => customField.name === customFieldName);

                    if (targetCustomFieldIndex !== -1) {
                        customFieldSetArray[targetCustomFieldSetPos].customFields.splice(targetCustomFieldIndex, 1);
                    }
                    if (customFieldSetArray[targetCustomFieldSetPos].customFields.length === 0) {
                        customFieldSetArray.splice(targetCustomFieldSetPos, 1);
                    }
                }
            }
        },

        checkCustomFieldInfo() {
            // Checks if the payment is ideal to add custom message display
            if (this.$route.params.id === 'ec8004c323c24809bea207eb69472ce0') {
                this.showNewAccountDetailCreationMessage = true;
            }
        },

        async loadAuthorizationMethodListener() {
            await this.$nextTick();
            const findAuthInput = () => {
                const authInput = document.querySelector('.sw-form-field-renderer-input-field__saferpay_shopware_authorization_method input');
                if (authInput) {
                    authInput.addEventListener('focus', this.addAuthListeners);
                    authInput.focus();
                    authInput.addEventListener('blur', this.addAuthListeners);
                    authInput.addEventListener('input', this.addAuthListeners);
                } else {
                    setTimeout(findAuthInput, 100);
                }
            }
            findAuthInput();
        },

        addAuthListeners() {
            const authListWidget = document.querySelector('.sw-select-option--widgetAuthorization');
            if (authListWidget) {
                authListWidget.addEventListener('focus', this.manageSaveCardOptions('widgetAuthorization'));
            }
            const authListPayment = document.querySelector('.sw-select-option--paymentPage');
            if (authListPayment) {
                authListPayment.addEventListener('focus', this.manageSaveCardOptions('paymentPage'));
            }
        },

        manageSaveCardOptions(item) {
            if (item == 'paymentPage') {
                let saveCardDom = document.querySelector('.sw-form-field-renderer-input-field__saferpay_shopware_save_card .sw-single-select__selection-text');
                saveCardDom.innerHTML = 'Inactive';
            }
        },


    },

    mounted() {
        this.loadAuthorizationMethodListener();
    },

    updated() {
        this.loadAuthorizationMethodListener();
    },
});
