import template from './sw-order-detail-saferpay.html.twig';
import './sw-order-detail-saferpay.scss';

const { Criteria } = Shopware.Data;

Shopware.Component.register('sw-order-detail-saferpay', {
    template,

    inject: [
        'repositoryFactory',
    ],

    data() {
        return {
            repository: null,
            history: null,
            transactions: null,
            showCaptureCancelBtn: false,
            capturePaymentTransaction: null,
            showRefundBtn: false,
            inProgress: false,
            inProgresssubmit: false,
            showModal: false,
            showModalCapture: false,
            showErrorResponse: false,
            showRefundModal: false,
            amountAlreadyRefund: 0,
            totalAvailableToRefund: 0,
            refundAmount: 0,
            reasonRefund: 0,
            alertSuccess: false,
            alertFail: false,
            totalAmount: 0,
            isLoading: false,
            transactionTab: this.$tc('saferpay.OrderDetails.transaction-tab'),
            transactionHistory: this.$tc('saferpay.OrderDetails.transaction-history'),
            errorResponseTab: this.$tc('saferpay.OrderDetails.error-response-tab'),
            refund: this.$tc('saferpay.OrderDetails.Refund'),
            confirmCapture: this.$tc('saferpay.OrderDetails.confirm-capture'),
            confirm: this.$tc('saferpay.OrderDetails.Confirm'),
            customerIdHidden: false,
            saferpayTokenHidden: false,
            saferpayCustomerIdHidden: false,
            transactionTypeHidden: false,
            preauthHidden: false,
            authorisationMethodHidden: false,
            transactionModeHidden: false,
            languageCodeHidden: false,
            paymentMethodHidden: false,
            authorizationAmountHidden: false,
            transactionIdHidden: false,
            currencyCodeHidden: false,
            acquirerNameHidden: false,
            acquirerReferenceHidden: false,
            sixTransactionReferenceHidden: false,
            approvalCodeHidden: false,
            recurringHidden: false,
            transactionStampHidden: false,
            settlementDateHidden: false,
            saferpayPaymentMethodHidden: false,
            saferpayMaskedNumberHidden: false,
            saferpayExpYearHidden: false,
            saferpayExpMonthHidden: false,
            saferpayCardCountryHidden: false,
            saferpayHolderNameHidden: false,
            aliasRegisterStatusHidden: false,
            liabilityShiftStatusHidden: false,
            liableEntityHidden: false,
            threeDsLiabilityShiftHidden: false,
            dccStatusHidden: false,
            dccAmountHidden: false,
            dccCurrencyCodeHidden: false,
            paidStatusHidden: false,
            transactionStatusHidden: false,
            activeHidden: false,
            captureIdHidden: false,
            captureAmountHidden: false            
        };
    },

    metaInfo() {
        return {
            title: 'Saferpay'
        };
    },

    computed: {
        saferpayTransactionHistoryRepository() {
            return this.repositoryFactory.create('saferpay_transaction_history');
        },
        saferpayPaymentTransactionRepository() {
            return this.repositoryFactory.create('saferpay_payment_transaction');
        },
        orderTransactionCaptureRepository() {
            return this.repositoryFactory.create('order_transaction_capture');
        }
    },

    created() {
        this.createdComponent();
        this.fetchTransactions();
    },

    mounted() {
        this.canRefund();
        this.canCaptureCancel();
        this.hasErrorResponse();
    },

    methods: {
        createdComponent() {
            try {
                const criteria = new Criteria();
                criteria.addFilter(Criteria.equals('saferpayPaymentTransaction.order.id', this.$route.params.id));
                this.saferpayTransactionHistoryRepository.search(
                    criteria,
                    Shopware.Context.api
                ).then((result) => {
                    this.history = result;
                });
            } catch (error) {
                console.error('Error fetching transactions:', error);
            }
        },

        fetchTransactions() {
            try {
                const criteria = new Criteria();
                criteria.addFilter(Criteria.equals('orderId', this.$route.params.id));

                this.saferpayPaymentTransactionRepository.search(criteria, Shopware.Context.api).then((result) => {
                    this.transactions = result[result.length - 1];
                    if (typeof this.transactions.preauth !== 'undefined') {
                        this.transactions.preauth = this.transactions.preauth ? 'Preauthorization' : 'Normal Authorization(final)';
                    }
                    if (typeof this.transactions.transactionMode !== 'undefined') {
                        this.transactions.transactionMode = this.transactions.transactionMode ? 'Live' : 'Test';
                    }
                    if (typeof this.transactions.recurring !== 'undefined') {
                        this.transactions.recurring = this.transactions.recurring ? 'Yes' : 'No';
                    }
                    if (typeof this.transactions.aliasRegisterStatus !== 'undefined') {
                        this.transactions.aliasRegisterStatus = this.transactions.aliasRegisterStatus ? 'Yes' : 'No';
                    }
                    if (typeof this.transactions.liabilityShiftStatus !== 'undefined') {
                        this.transactions.liabilityShiftStatus = this.transactions.liabilityShiftStatus ? 'Authenticated' : 'Rejected';
                    }
                    if (typeof this.transactions.threeDsLiabilityShift !== 'undefined') {
                        this.transactions.threeDsLiabilityShift = this.transactions.threeDsLiabilityShift ? 'Authenticated' : 'Rejected';
                    }
                    if (typeof this.transactions.dccStatus !== 'undefined') {
                        this.transactions.dccStatus = this.transactions.dccStatus ? 'Yes' : 'No';
                    }
                    if (typeof this.transactions.paidStatus !== 'undefined') {
                        this.transactions.paidStatus = this.transactions.paidStatus ? 'Yes' : 'No';
                    }
                    if (typeof this.transactions.transactionStatus !== 'undefined') {
                        this.transactions.transactionStatus = this.transactions.transactionStatus ? 'Yes' : 'No';
                    }
                    if (typeof this.transactions.active !== 'undefined') {
                        this.transactions.active = this.transactions.active ? 'Yes' : 'No';
                    } 

                    if (this.transactions.customerId == null) {
                        this.customerIdHidden = true;
                    }
                    if (this.transactions.saferpayToken == null) {
                        this.saferpayTokenHidden = true;
                    }
                    if (this.transactions.saferpayCustomerId == null) {
                        this.saferpayCustomerIdHidden = true;
                    }
                    if (this.transactions.transactionType == null) {
                        this.transactionTypeHidden = true;
                    }
                    if (this.transactions.preauth == null) {
                        this.preauthHidden = true;
                    }
                    if (this.transactions.authorisationMethod == null) {
                        this.authorisationMethodHidden = true;
                    }
                    if (this.transactions.transactionMode == null) {
                        this.transactionModeHidden = true;
                    }
                    if (this.transactions.languageCode == null) {
                        this.languageCodeHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_payment_method == null) {
                        this.paymentMethodHidden = true;
                    }
                    if (this.transactions.authorizationAmount == null) {
                        this.authorizationAmountHidden = true;
                    }
                    if (this.transactions.orderTransactionId == null) {
                        this.transactionIdHidden = true;
                    }
                    if (this.transactions.currencyCode == null) {
                        this.currencyCodeHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.acquirer_name == null) {
                        this.acquirerNameHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.acquirer_reference == null) {
                        this.acquirerReferenceHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.six_transaction_reference == null) {
                        this.sixTransactionReferenceHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.approval_code == null) {
                        this.approvalCodeHidden = true;
                    }
                    if (this.transactions.recurring == null) {
                        this.recurringHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.transaction_stamp == null) {
                        this.transactionStampHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.settlement_date == null) {
                        this.settlementDateHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_payment_method == null) {
                        this.saferpayPaymentMethodHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_masked_number == null) {
                        this.saferpayMaskedNumberHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_exp_year == null) {
                        this.saferpayExpYearHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_exp_month == null) {
                        this.saferpayExpMonthHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_card_country == null) {
                        this.saferpayCardCountryHidden = true;
                    }
                    if (this.transactions.saferpayPaymentInfo.saferpay_holder_name == null) {
                        this.saferpayHolderNameHidden = true;
                    }
                    if (this.transactions.aliasRegisterStatus == null) {
                        this.aliasRegisterStatusHidden = true;
                    }
                    if (this.transactions.liabilityShiftStatus == null) {
                        this.liabilityShiftStatusHidden = true;
                    }
                    if (this.transactions.liableEntity == null) {
                        this.liableEntityHidden = true;
                    }
                    if (this.transactions.threeDsLiabilityShift == null) {
                        this.threeDsLiabilityShiftHidden = true;
                    }
                    if (this.transactions.dccStatus == null) {
                        this.dccStatusHidden = true;
                    }
                    if (this.transactions.dccInfo.dcc_amount == null) {
                        this.dccAmountHidden = true;
                    }
                    if (this.transactions.dccInfo.dcc_currency_code == null) {
                        this.dccCurrencyCodeHidden = true;
                    }
                    if (this.transactions.paidStatus == null) {
                        this.paidStatusHidden = true;
                    }
                    if (this.transactions.transactionStatus == null) {
                        this.transactionStatusHidden = true;
                    }
                    if (this.transactions.active == null) {
                        this.activeHidden = true;
                    }
                    if (this.transactions.captureId == null) {
                        this.captureIdHidden = true;
                        if(transactions.capturedAmount == null){
                            this.captureAmountHidden = true;
                        }
                    }             
 
                    this.totalAmount = this.transactions.capturedAmount;
                });
            } catch (error) {
                console.error('Error fetching transactions:', error);
                // Handle the error as per your requirement (e.g., show an error message).
            }
        },

        canRefund() {
            const refundCriteria = new Criteria();
            refundCriteria.addFilter(Criteria.equals('saferpayPaymentTransaction.order.id', this.$route.params.id));
            refundCriteria.addFilter(Criteria.equals('transactionStatus', 'CAPTURED'));
            this.saferpayTransactionHistoryRepository.search(refundCriteria, Shopware.Context.api).then((refundResult) => {
                this.showRefundBtn = typeof refundResult[0] !== 'undefined';
            });
        },

        canCaptureCancel() {
            const checkCriteria = new Criteria();
            checkCriteria.addFilter(Criteria.equals('saferpayPaymentTransaction.order.id', this.$route.params.id));

            this.saferpayTransactionHistoryRepository.search(checkCriteria, Shopware.Context.api).then((checkResult) => {
                const lastTransactionStatus = checkResult[checkResult.length - 1].transactionStatus;
                const authorizedTransaction = checkResult.find(item => item.transactionStatus === 'AUTHORIZED');
                const capturedTransaction = checkResult.find(item => item.transactionStatus === 'CAPTURED');
                const cancelTransaction = checkResult.find(item => item.transactionStatus === 'CANCEL');

                if (lastTransactionStatus === 'AUTHORIZED' || (authorizedTransaction && !capturedTransaction && !cancelTransaction)) {
                    this.showCaptureCancelBtn = true;
                    this.capturePaymentTransaction = authorizedTransaction;
                } else {
                    this.showCaptureCancelBtn = false;
                }
            });
        },

        hasErrorResponse() {
            const errorCriteria = new Criteria();
            errorCriteria.addFilter(Criteria.equals('saferpayPaymentTransaction.order.id', this.$route.params.id));

            this.saferpayTransactionHistoryRepository.search(errorCriteria, Shopware.Context.api).then((errorResult) => {
                const cancelTransaction = errorResult.find(item => item.transactionStatus === 'CANCEL');

                if (cancelTransaction) {
                    this.showErrorResponse = true;
                } else {
                    this.showErrorResponse = false;
                }
            });
        },

        handleCapture() {
            this.inProgress = true;
            this.showModalCapture = false;
            try {
                const captureUrl = '/api/v' + Shopware.Context.api.apiVersion + '/_action/saferpay-shopware/manual-capture/' + this.capturePaymentTransaction.paymentTransactionId;
                const headersArray = {
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + Shopware.Context.api.authToken.access,
                };
                fetch(captureUrl, {
                    method: 'GET',
                    credentials: 'same-origin',
                    headers: headersArray,
                })
                    .then(response => response.json())
                    .then(data => {
                        const message = data.success;
                        if (message === true) {
                            this.showCaptureCancelBtn = false;
                            this.showRefundBtn = true;
                        } else {
                            this.showCaptureCancelBtn = true;
                            this.inProgress = false;
                            this.showRefundBtn = false;
                        }
                        this.createdComponent();
                        this.fetchTransactions();
                    })
                    .catch(error => {
                        console.error('Error calling backend controller:', error);
                    });
            } catch (error) {
                console.log(error);
            }
        },

        onConfirmCapture() {
            this.showModalCapture = true;
        },

        onCloseConfirmCaptureModal() {
            this.showModalCapture = false;
        },

        onConfirmCancel() {
            this.showModal = true;
        },

        confirmCancellation() {
            this.inProgress = true;
            this.showModal = false;
            try {
                const captureUrl = '/api/v' + Shopware.Context.api.apiVersion + '/_action/saferpay-shopware/manual-cancellation/' + this.capturePaymentTransaction.paymentTransactionId;
                const headersArray = {
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + Shopware.Context.api.authToken.access,
                };
                fetch(captureUrl, {
                    method: 'GET',
                    credentials: 'same-origin',
                    headers: headersArray,
                })
                    .then(response => response.json())
                    .then(data => {
                        const message = data.success;
                        if (message === true) {
                            this.showCaptureCancelBtn = false;
                        } else {
                            this.showCaptureCancelBtn = true;
                            this.inProgress = false;
                        }
                        this.createdComponent();
                        this.fetchTransactions();
                    })
                    .catch(error => {
                        console.error('Error calling backend controller:', error);
                    });
            } catch (error) {
                console.log(error);
            }
        },

        onCloseConfirmModal() {
            this.showModal = false;
        },

        openRefundModal() {
            this.showRefundModal = true;
            this.alertSuccess = false;
            this.alertFail = false;
            this.inProgress = false;

            const refundDataCriteria = new Criteria();
            refundDataCriteria.addFilter(Criteria.equals('orderTransactionId', this.transactions.orderTransactionId));
            this.orderTransactionCaptureRepository.search(refundDataCriteria, Shopware.Context.api).then((orderTransactionCaptureResult) => {
                orderTransactionCaptureResult.map((otc) => {
                    this.amountAlreadyRefund += otc.amount.totalPrice;
                })
                this.amountAlreadyRefund = parseFloat(this.amountAlreadyRefund).toFixed(2);
                this.totalAvailableToRefund = this.transactions.capturedAmount - this.amountAlreadyRefund;
                this.totalAvailableToRefund = parseFloat(this.totalAvailableToRefund).toFixed(2)

            });

        },

        closeRefundModal() {
            this.showRefundModal = false;
            this.amountAlreadyRefund = 0;
        },

        confirmRefund() {
            this.amountAlreadyRefund = 0;
            this.isLoading = true;
            this.inProgresssubmit = true;
            try {
                // Get the values of the input elements using their IDs
                // var restockRefundItem = document.getElementsByName('restock-refunded-item');
                // const checkbox = document.querySelector('#restock-refunded-item');
                const amountAlreadyRefund = document.getElementsByName('amount-already-refund');
                const totalAvailableRefund = document.getElementsByName('total-available-refund');
                const totalAvailableToRefund = document.getElementsByName('total-available-to-refund');
                const refundAmount = document.getElementsByName('refund-amount');
                const reasonRefund = document.getElementsByName('reason-refund');

                var restockRefundItemValue = false;

                const amountAlreadyRefundValue = amountAlreadyRefund.length > 0 ? amountAlreadyRefund[0].value : '';
                const totalAvailableRefundValue = totalAvailableRefund.length > 0 ? totalAvailableRefund[0].value : '';
                const totalAvailableToRefundValue = totalAvailableToRefund.length > 0 ? totalAvailableToRefund[0].value : '';
                const refundAmountValue = refundAmount.length > 0 ? refundAmount[0].value : '';
                const reasonRefundValue = reasonRefund.length > 0 ? reasonRefund[0].value : '';
                const currencyCode = this.transactions.currencyCode;
                const orderId = this.transactions.orderId;

                // Create a plain object with the form data
                var formData = {
                    'restockRefundItem': restockRefundItemValue,
                    'amountAlreadyRefund': amountAlreadyRefundValue,
                    'totalAvailableRefund': totalAvailableRefundValue,
                    'totalAvailableToRefund': this.totalAvailableToRefund,
                    'refundAmount': refundAmountValue,
                    'reasonRefund': reasonRefundValue,
                    'captureId': this.transactions.captureId,
                    'currencyCode': currencyCode,
                    'orderId': orderId,
                    'orderTransactionId': this.transactions.orderTransactionId
                };


                const captureUrl = '/api/_action/saferpay-shopware/refund-transaction';
                const headersArray = {
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + Shopware.Context.api.authToken.access,
                };
                fetch(captureUrl, {
                    method: 'POST',
                    credentials: 'same-origin',
                    headers: headersArray,
                    body: JSON.stringify(formData),
                })
                    .then(response => response.json())
                    .then(data => {
                        this.refundAmount = 0;
                        this.inProgresssubmit = false;
                        this.isLoading = false;
                        const status = data.success;
                        if (status) {
                            this.alertSuccess = true;

                        } else {
                            this.alertFail = true;
                        }
                        this.createdComponent();
                        this.fetchTransactions();
                        const refundDataCriteria = new Criteria();
                        refundDataCriteria.addFilter(Criteria.equals('orderTransactionId', this.transactions.orderTransactionId));
                        this.orderTransactionCaptureRepository.search(refundDataCriteria, Shopware.Context.api).then((orderTransactionCaptureResult) => {
                            orderTransactionCaptureResult.map((otc) => {
                                this.amountAlreadyRefund += otc.amount.totalPrice;

                            })
                            this.amountAlreadyRefund = parseFloat(this.amountAlreadyRefund).toFixed(2);
                            this.totalAvailableToRefund = this.transactions.capturedAmount - this.amountAlreadyRefund;
                            this.totalAvailableToRefund = parseFloat(this.totalAvailableToRefund).toFixed(2)
                        });
                        setTimeout(() => {
                            this.alertSuccess = false;
                            this.alertFail = false;
                        }, 5000);
                    })
                    .catch(error => {
                        this.isLoading = false;
                        this.inProgresssubmit = false;
                        console.error('Error calling backend controller:', error);
                    });

            } catch (error) {
                this.isLoading = false;
                this.inProgresssubmit = false;
                console.log(error);
            }


        }

    }
});