import Plugin from 'src/plugin-system/plugin.class';
import HttpClient from 'src/service/http-client.service';
var isFieldsValid = {
    holdername: false,
    cardnumber: false,
    expiration: false,
    cvc: true
}
var token = -1;

export default class SaferpayFieldsSaveCardCustom extends Plugin {

    data() {
        return {
            isIframeLoading: null,
            saferpayToken: null,
            ajaxProcess: false,
            saferpayFieldsProcess: false,
        }
    }
    
    init() {
        this.creditCardDiv = this.el.querySelector('saferpay-div');
        this._client = new HttpClient();
        this.button = this.el.children['addCardBtn'];
        this.iframe = document.querySelector("#spsw-iframe");
        this.iframeLoader = document.querySelector("#ajax-loading-payment");
        this.saferpayFieldsProcess = false;
        if (typeof creditParams.cardTemplate !== 'undefined') {

            let labels = {
                invalid: creditParams.labels.invalid,
                empty: creditParams.labels.empty,
                unsupported: creditParams.labels.unsupported,
                expired: creditParams.labels.expired
            };

            SaferpayFields.init({
                accessToken: creditParams.accessToken,
                url: creditParams.sourceUrl,
                placeholders: {
                    holdername: creditParams.cardHolderName,
                    cardnumber: '0000 0000 0000 0000',
                    expiration: creditParams.expiration,
                    cvc: '000'
                },
                paymentMethods: creditParams.paymentMethods,
                style: this.getTemplateStyle(),
                cssUrl: creditParams.cssUrl,
                onSuccess: (evt) => {
                    this.addToken('successCallback');
                },

                onError: (evt) => {
                    this.addToken('errorCallback');
                },

                onBlur: (evt) => {
                    //Callback on blur (Card Holder leaves field)
                },

                onValidated: (evt) => {
                    const elemIds = {
                        cardnumber: 'card-number-help',
                        expiration: 'expiration-help',
                        holdername: 'holder-name-help',
                        cvc: 'cvc-help'
                    };
                    const successElem = document.querySelectorAll('.saferpay-saved-cards .success-alert')[0];
                    successElem ? successElem.remove() : '';
                    const failElem = document.querySelectorAll('.saferpay-saved-cards > .error-alert')[0];
                    failElem ? failElem.remove() : '';
                    const elemId = elemIds[evt.fieldType];
                    if (!elemId) return;

                    isFieldsValid[evt.fieldType] = evt.isValid;
                
                    const elem = document.getElementById(elemId);
                    const isSample3 = creditParams.cardTemplate === 'sample3';
                    
                    if (!isSample3) {
                        const message = labels[evt.reason || 'invalid'];
                        this.updateValidationMessage(elem, evt.isValid, message);
                    }
                    
                    this.setFieldValidationStatus(evt.id, evt.isValid);
                    this.addToken('validateCallback');
                },

                onFocus: function (e) {
                    let pictureCard = document.querySelector("#picture .saferpaycard");
                    if (pictureCard) {
                        switch (e.fieldType) {
                            case 'cvc':
                                pictureCard.classList.add('show-back', 'cvc');
                                pictureCard.classList.remove('cardnumber', 'expiration');
                                break;
                            case 'cardnumber':
                                pictureCard.classList.add('cardnumber');
                                pictureCard.classList.remove('show-back', 'expiration', 'cvc');
                                break;
                            case 'expiration':
                                pictureCard.classList.add('expiration');
                                pictureCard.classList.remove('show-back', 'cardnumber', 'cvc');
                                break;
                            default:
                                break;
                        }
                    }                    
                }
            });
            document.querySelector('#saferpay-div').style = 'opacity:1';
        }
        this.addCard();
    }

    addCard(){
        document.querySelector('#addCardBtn').addEventListener('click', (event) => {
            let cardType = document.querySelector('#cardTypeSelect').value;
            if (cardType == 'cards') {
                this.validateSaferpayFieldsToken();
                this.submitSaferpayFields();
            } else {
                this.addOtherCard();
            }
        });
    }

    updateValidationMessage(elem, isValid, message) {
        if (!isValid) elem.innerHTML = message;
        else elem.innerHTML = '';
    }
    
    setFieldValidationStatus(fieldId, isValid) {
        document.querySelector(`#${fieldId}`).setAttribute('data-isvalid', isValid.toString());
    }

    getTemplateStyle(){
        let inputExpiryWidth ="100%"; 
        let cardnumberWidth = "100%";
        var styleDesign;
        switch (creditParams.cardTemplate) {
            case 'sample1': styleDesign = {
                '.form-control': 'border: solid 1px #CFCFCF; border-radius: 0px; height: 40px !important; width: calc(100% - 1px);',

                '.form-control.is-valid, .was-validated .form-control:valid': 'border-color: #28a745;padding-right: calc(1.5em + .75rem); background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHZpZXdCb3g9JzAgMCA4IDgnPjxwYXRoIGZpbGw9JyMyOGE3NDUnIGQ9J00yLjMgNi43M0wuNiA0LjUzYy0uNC0xLjA0LjQ2LTEuNCAxLjEtLjhsMS4xIDEuNCAzLjQtMy44Yy42LS42MyAxLjYtLjI3IDEuMi43bC00IDQuNmMtLjQzLjUtLjguNC0xLjEuMXonLz48L3N2Zz4=);background-repeat: no-repeat; background-position: 83% 50%; background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '.form-control.is-invalid, .was-validated .form-control:invalid': '  border-color: #dc3545;    padding-right: calc(1.5em + .75rem);background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGZpbGw9JyNkYzM1NDUnIHZpZXdCb3g9Jy0yIC0yIDcgNyc+PHBhdGggc3Ryb2tlPScjZGMzNTQ1JyBkPSdNMCAwbDMgM20wLTNMMCAzJy8+PGNpcmNsZSByPScuNScvPjxjaXJjbGUgY3g9JzMnIHI9Jy41Jy8+PGNpcmNsZSBjeT0nMycgcj0nLjUnLz48Y2lyY2xlIGN4PSczJyBjeT0nMycgcj0nLjUnLz48L3N2Zz4=);background-repeat: no-repeat;background-position: 83% 50%;    background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '.form-control.is-valid.is-pristine': 'background-image: inherit; border-color:#CFCFCF;',

                '.input-group .logo': 'right: 2em;',

                '.expiration__input': 'padding: 0.3125rem 0.3125rem 0.125rem 0.8125rem;',

                '.cardnumber__logos': 'right: 15px;',

                '#fields-holder-name': 'width:99%; display:block;',

                '#fields-card-number': 'width:99%; display:block;'
            }; break;
            case 'sample2': styleDesign = {
                '.form-control': ' border: solid 1px #CFCFCF; border-radius: 8px; height: 46px !important',

                '.form-control.is-valid, .was-validated .form-control:valid': 'border-color: #28a745;padding-right: calc(1.5em + .75rem); background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHZpZXdCb3g9JzAgMCA4IDgnPjxwYXRoIGZpbGw9JyMyOGE3NDUnIGQ9J00yLjMgNi43M0wuNiA0LjUzYy0uNC0xLjA0LjQ2LTEuNCAxLjEtLjhsMS4xIDEuNCAzLjQtMy44Yy42LS42MyAxLjYtLjI3IDEuMi43bC00IDQuNmMtLjQzLjUtLjguNC0xLjEuMXonLz48L3N2Zz4=);background-repeat: no-repeat; background-position: calc(100% - 34px) 50%; background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '.form-control.is-invalid, .was-validated .form-control:invalid': '  border-color: #dc3545;    padding-right: calc(1.5em + .75rem);background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGZpbGw9JyNkYzM1NDUnIHZpZXdCb3g9Jy0yIC0yIDcgNyc+PHBhdGggc3Ryb2tlPScjZGMzNTQ1JyBkPSdNMCAwbDMgM20wLTNMMCAzJy8+PGNpcmNsZSByPScuNScvPjxjaXJjbGUgY3g9JzMnIHI9Jy41Jy8+PGNpcmNsZSBjeT0nMycgcj0nLjUnLz48Y2lyY2xlIGN4PSczJyBjeT0nMycgcj0nLjUnLz48L3N2Zz4=);background-repeat: no-repeat;background-position: 83% 50%;    background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '#fields-card-number.is-invalid ~ span.logo': 'right: 28px',

                '#fields-card-number.is-valid ~ span.logo': 'right: 28px',

                '.form-control.is-valid.is-pristine': 'background-image: inherit; border-color:#CFCFCF;'
            }; break;
            case 'sample3': styleDesign = {
                '.form-control': 'background-color: transparent;color: #fff;border-radius: 0px;border:none;outline: none;',

                '.form-control::-webkit-input-placeholder': 'color:#becff2',

                '.form-control::-moz-placeholder': 'color:#becff2',

                '.form-control:-ms-input-placeholder': 'color:#becff2',

                '.form-control.input-verificationcode': 'text-align: left',

                '.form-control.is-valid, .was-validated .form-control:valid': 'border-color: #28a745;padding-right: calc(1.5em + .75rem); background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHZpZXdCb3g9JzAgMCA4IDgnPjxwYXRoIGZpbGw9JyMyOGE3NDUnIGQ9J00yLjMgNi43M0wuNiA0LjUzYy0uNC0xLjA0LjQ2LTEuNCAxLjEtLjhsMS4xIDEuNCAzLjQtMy44Yy42LS42MyAxLjYtLjI3IDEuMi43bC00IDQuNmMtLjQzLjUtLjguNC0xLjEuMXonLz48L3N2Zz4=);background-repeat: no-repeat; background-position: 83% 50%; background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '.form-control.is-invalid, .was-validated .form-control:invalid': '  border-color: #dc3545;    padding-right: calc(.75em + .375rem);background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGZpbGw9JyNkYzM1NDUnIHZpZXdCb3g9Jy0yIC0yIDcgNyc+PHBhdGggc3Ryb2tlPScjZGMzNTQ1JyBkPSdNMCAwbDMgM20wLTNMMCAzJy8+PGNpcmNsZSByPScuNScvPjxjaXJjbGUgY3g9JzMnIHI9Jy41Jy8+PGNpcmNsZSBjeT0nMycgcj0nLjUnLz48Y2lyY2xlIGN4PSczJyBjeT0nMycgcj0nLjUnLz48L3N2Zz4=);background-repeat: no-repeat;background-position: 83% 50%;    background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '.form-control.is-valid.is-pristine': 'background-image: inherit; border-color:#CFCFCF;',

                '#fields-expiration': 'min-width: auto;',

                'input.input-cardnumber': `width: ${cardnumberWidth}`,

                'input.input-expiry': 'width: 100%; padding-right: calc(.75em + .375rem) !important;background-position: 100% 50% !important;',
                '.cardnumber__logos': 'right: 0px;',
            }; break;
            case 'sample4': styleDesign = {
                '.form-control.is-valid, .was-validated .form-control:valid': 'border-color: #28a745;padding-right: calc(1.5em + .75rem); background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHZpZXdCb3g9JzAgMCA4IDgnPjxwYXRoIGZpbGw9JyMyOGE3NDUnIGQ9J00yLjMgNi43M0wuNiA0LjUzYy0uNC0xLjA0LjQ2LTEuNCAxLjEtLjhsMS4xIDEuNCAzLjQtMy44Yy42LS42MyAxLjYtLjI3IDEuMi43bC00IDQuNmMtLjQzLjUtLjguNC0xLjEuMXonLz48L3N2Zz4=);background-repeat: no-repeat; background-position:calc(100% - 34px) 50%; background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '#fields-card-number': 'padding-left: 0',

                '#fields-card-number ~ span.logo': 'position: absolute; left: 10px;',

                '.form-control.is-invalid, .was-validated .form-control:invalid': '  border-color: #dc3545;    padding-right: calc(1.5em + .75rem);background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGZpbGw9JyNkYzM1NDUnIHZpZXdCb3g9Jy0yIC0yIDcgNyc+PHBhdGggc3Ryb2tlPScjZGMzNTQ1JyBkPSdNMCAwbDMgM20wLTNMMCAzJy8+PGNpcmNsZSByPScuNScvPjxjaXJjbGUgY3g9JzMnIHI9Jy41Jy8+PGNpcmNsZSBjeT0nMycgcj0nLjUnLz48Y2lyY2xlIGN4PSczJyBjeT0nMycgcj0nLjUnLz48L3N2Zz4=);background-repeat: no-repeat;background-position: 83% 50%;    background-size: calc(.75em + .375rem) calc(.75em + .375rem);',

                '.form-control.is-valid.is-pristine': 'background-image: inherit; border-color:#CFCFCF;',

                'input#fields-card-number': 'padding-left: .75rem;',

                '.input-expiry': 'min-height: 36px;',

                '.input-large': 'width:99%;',
                '.form-control.input-verificationcode': 'background-position: calc(100% - 4px) 50% !important;padding-right: calc(1em + .375rem) !important;',
                'input.input-expiry': 'padding-right: calc(1em + .375rem) !important;background-position: calc(100% - 4px) 50% !important;',
                '.cardnumber__logos': 'right: 3px;',
            }; break;
            default: styleDesign = {}; break;
        }
        return styleDesign;
    }
    
    validateSaferpayFieldsToken() {
        if (!document.querySelector('#token') || document.querySelector('#token').value == -1) {
            let errorDiv = document.querySelector('#credit-card-error-div');
            errorDiv.innerHTML = '<p>' + creditParams.noDetails + '</p>';
            errorDiv.parentElement.parentElement.parentElement.scrollIntoView({ behavior: 'smooth' });
            errorDiv.parentElement.parentElement.parentElement.focus();
            return false;
        }
    }

    addToken(data) {
        var creditForm = document.querySelector('div#credit-card-section');
        if (data != 'errorCallback') {
            var isFieldsValidTemp = Object.values(isFieldsValid).some(value => value === false);
            if (!isFieldsValidTemp) {
                token = 1;
                document.querySelector('#addCardBtn').setAttribute("data-bs-toggle", "modal");
            }
            else {
                token = -1;
                document.querySelector('#addCardBtn').removeAttribute("data-bs-toggle", "modal");
            }

        } else {
            document.querySelector('#addCardBtn').removeAttribute("data-bs-toggle");
        }

        if (!document.querySelector('#token')) {
            var tokenInput = document.createElement('input');
            tokenInput.setAttribute('class', 'form-control');
            tokenInput.setAttribute('id', 'token');
            tokenInput.setAttribute('name', 'token');
            tokenInput.setAttribute('value', token);
            tokenInput.setAttribute('type', 'hidden');
            creditForm.appendChild(tokenInput);
        } else if (document.querySelector('#token') && (document.querySelector('#token').value == '1' || document.querySelector('#token').value == '-1')) {
            document.querySelector('#token').setAttribute('value', token);
            if (document.querySelector('#token').value == '1') 
                document.querySelector(".credit-card-error-div").style = "display:none;";
        }
        return true;
    }

    submitSaferpayFields() {
        SaferpayFields.submit({
            onSuccess: (evt) => {
                let token = evt.token;
                let formData = { 'saferpayFieldsAccessToken': token };
                this._client.post('/saferpay/account/saved/card/add/card', JSON.stringify(formData), (response) => {
                    this.setIframeContent(response);
                }, 'application/json', true);
                return false;
            },

            onError: (evt) => {
                if (!document.querySelector('#token')) {
                    var tokenInput = document.createElement('input');
                    tokenInput.setAttribute('class', 'form-control');
                    tokenInput.setAttribute('id', 'token');
                    tokenInput.setAttribute('name', 'token');
                    tokenInput.setAttribute('value', token);
                    tokenInput.setAttribute('type', 'hidden');
                    creditForm.appendChild(tokenInput);
                } else if (document.querySelector('#token') && (document.querySelector('#token').value == '1' || document.querySelector('#token').value == '-1')) {
                    document.querySelector('#token').setAttribute('value', token);
                    if (document.querySelector('#token').value == '1')
                        document.querySelector(".credit-card-error-div").style = "display:none;";
                }
                return false;
            }

        });
    }

    setIframeContent(data) {
        try {
            let jsonResp = JSON.parse(data);
            if (jsonResp.cardType == 'postfinance') {
                window.location.href = jsonResp.iframeUrl;
            }
            if (jsonResp.redirect == 'no') {
                this._client.get(
                    jsonResp.iframeUrl,
                    (response) => {
                        console.log(response);
                        window.location.reload();
                    }
                );
            } else {
                this.iframe.src = jsonResp.iframeUrl;
                this.saferpayToken = jsonResp.saferpayToken;
                this.isIframeLoading = false;
                this.iframeLoader.classList.remove('ajax-loading-payment');
            }
        } catch(error) {
            console.log('error in try');
            window.location.href = window.location.href;
        }
    }


    addOtherCard() {
        let formData = {};
        this._client.post('/saferpay/account/saved/card/add/card/', JSON.stringify(formData), (response) => {
            this.setIframeContent(response);
        }, 'application/json', true);
        return false;
    }

}