import HttpClient from 'src/service/http-client.service';
import Plugin from 'src/plugin-system/plugin.class';

export default class AjaxSaferpayIframePlugin extends Plugin {

    data() {
        return {
            isIframeLoading: null,
            orderId: null,
            ajaxProcess: false,
            saferpayFieldsProcess: false,
            iframeInitialize:false,
        }
    }

    init() {
        this._client = new HttpClient();
        this.button = this.el.children['saferpayIframeBtn'];
        this.iframe = this.el.querySelector("#spsw-iframe");
        this.iframeLoader = this.el.querySelector("#ajax-loading-payment");
        this.saferpayFieldsProcess = false;
        this.iframeInitialize = false;
        this.pageLoadEvents();
    }

    checkIframeContent() {
        if (!this.orderId) {
            return false;
        }
        try {
            if (!this.ajaxProcess) {
                this.ajaxProcess = true;
                this._client.get(
                    '/saferpay/transaction/order/status/' + this.orderId,
                    (response) => {
                        let resp = JSON.parse(response);
                        if (resp.paymentState === 'Paid' || resp.paymentState === 'Unconfirmed') {
                            window.location.href = window.router['frontend.checkout.finish.page'] + '?orderId=' + this.orderId;
                        }
                        if (resp.redirectTo) {
                            if (resp.redirectStatus === 'cancel') {
                                this.isIframeLoading = true;
                                this.iframe.src = "";
                                window.location.href = resp.errorUrl;
                            }
                        }
                        this.ajaxProcess = false;
                    }
                );
            }
        } catch (e) {
            this.ajaxProcess = false;
            console.log(e);
        }
    }

    onTermsAccept() {
        this.checkModalDisplay();
    }

    checkModalDisplay() {
        if (
            (typeof document.forms.confirmOrderForm.elements.tos === "undefined")
            || (document.forms.confirmOrderForm.elements.tos === null)
        ) {
            if (this.checkModalDisplayForCreditCard()) {
                this.button.removeAttribute('data-bs-toggle', 'modal');
                return false;
            }
            this.button.setAttribute('data-bs-toggle', 'modal');
            this.button.onclick = this._fetch.bind(this);
        } else if (document.forms.confirmOrderForm.elements.tos.checked) {
            if (this.checkModalDisplayForCreditCard()) {
                this.button.removeAttribute('data-bs-toggle', 'modal');
                return false;
            }
            this.button.setAttribute('data-bs-toggle', 'modal');
            this.button.onclick = this._fetch.bind(this);
        } else {
            this.button.removeAttribute('data-bs-toggle', 'modal');
        }
    }

    pageLoadEvents() {
        this.checkModalDisplay();
        if ((typeof document.forms.confirmOrderForm.elements.tos != 'undefined') && (document.forms.confirmOrderForm.elements.tos != null)) {
            document.forms.confirmOrderForm.elements.tos.addEventListener('click', this.onTermsAccept.bind(this));
            document.getElementById('saferpayIframeBtn').addEventListener('click', (ele) => {
                if (!document.forms.confirmOrderForm.elements.tos.checked) {
                    document.forms.confirmOrderForm.elements.tos.focus();
                    document.forms.confirmOrderForm.elements.tos.reportValidity();
                    return false;
                }
                if (typeof ele.delegateTarget == 'undefined') {
                    if (this.checkIfCreditCardPayment() && !this.checkIfAliasSelected()) {
                        this.validateSaferpayFieldsToken();
                    }
                } else {
                    this._fetch();
                }
            });
        } else {
            document.getElementById('saferpayIframeBtn').addEventListener('click', (ele) => {
                if (typeof ele.delegateTarget == 'undefined') {
                    if (this.checkIfCreditCardPayment() && !this.checkIfAliasSelected()) {
                        this.validateSaferpayFieldsToken();
                    }
                } else {
                    this._fetch();
                }
            });
        }
        var savedCardId = document.getElementById('customer_selected_alias');
        savedCardId && savedCardId.addEventListener('change', (e) => {
            if (this.checkIfCreditCardPayment()) {
                if (
                    (typeof document.forms.confirmOrderForm.elements.tos !== "undefined")
                    && (document.forms.confirmOrderForm.elements.tos !== null)
                ) {
                    if (document.forms.confirmOrderForm.elements.tos.checked) {
                        if (e.target.value != '' && e.target.value != null){
                              document.getElementById('saferpayIframeBtn').setAttribute('data-bs-toggle', 'modal');
                        } else {                                 
                            if (this.checkIfCreditCardPayment() && (!document.querySelector('#token')
                                 || document.querySelector('#token').value == -1)) {
                                document.getElementById('saferpayIframeBtn').removeAttribute('data-bs-toggle', 'modal');
                            } else {
                                document.getElementById('saferpayIframeBtn').setAttribute('data-bs-toggle', 'modal');
                            }
                        }                        
                    }
                    else {
                        document.getElementById('saferpayIframeBtn').removeAttribute('data-bs-toggle', 'modal');
                    }
                }
            }
        });
    } 

    _fetch() {
        this.isIframeLoading = true;
        if (this.checkIfCreditCardPayment() && !this.checkIfAliasSelected()) {
            this.validateSaferpayFieldsToken();
            if (document.getElementById('tos') == null) {
                if (document.querySelector('#token').value == 1 && !this.saferpayFieldsProcess) {
                    this.saferpayFieldsProcess = true;
                    this.submitSaferpayFields();
                }                
            }
            else if (document.getElementById('tos').checked) {
                if (document.querySelector('#token').value == 1 && !this.saferpayFieldsProcess) {
                    this.saferpayFieldsProcess = true;
                    this.submitSaferpayFields();
                }
            }

        } else {
            if(!this.iframeInitialize){
            this._client.post('/saferpay/transaction/initialize/iframe', new FormData(document.getElementById('confirmOrderForm')), this.setIframeContent.bind(this), 'application/json', true);
            }
            this.iframeInitialize = true;
        }
    }

    setIframeContent(data) {
        let jsonResp = JSON.parse(data);
        if (jsonResp.noRedirect) {
        } else {
            this.iframe.src = jsonResp.iframeUrl;
            this.orderId = jsonResp.orderId;
            this.isIframeLoading = false;
            this.iframeLoader.classList.remove('ajax-loading-payment');
        }
        if ((typeof document.getElementById('customer_selected_alias') !== 'undefined') && (document.getElementById('customer_selected_alias') != null)) {
            this.checkIframeContentInterval = setInterval(() => {
                this.checkIframeContent();
            }, 1000);
        }
        if (this.checkIfCreditCardPayment() && !this.checkIfAliasSelected()) {
            this.checkIframeContentInterval = setInterval(() => {
                this.checkIframeContent();
            }, 1000);
        }
        window.addEventListener('message', this.handleIframeMessage.bind(this));
    }

    handleIframeMessage(event) {
        if (event.origin !== 'https://test.saferpay.com' || event.origin !== 'https://www.saferpay.com') {
            console.log('message from saferpay');
            this.checkIframeContentInterval = setInterval(() => {
                this.checkIframeContent();
            }, 1000);
        } else {
            console.log('message from other');
            window.location.reload();
        }
    }

    checkModalDisplayForCreditCard() {
        if (this.checkIfCreditCardPayment() && !this.checkIfAliasSelected() && (!document.querySelector('#token') || document.querySelector('#token').value == -1)) {
            return true;
        }
        return false;
    }

    checkIfAliasSelected() {
        if ((typeof document.getElementById('customer_selected_alias') !== 'undefined') && (document.getElementById('customer_selected_alias') != null) && (document.getElementById('customer_selected_alias').value != '')) {
            return true;
        }
        return false;
    }

    checkIfCreditCardPayment() {
        let btn = this.button;
        if (typeof this.button === 'undefined') {
            btn = document.getElementById('saferpayIframeBtn');
        }
        if (btn.getAttribute('data-credit-card') && btn.getAttribute('data-credit-card') === 'yes') {
            return true;
        }
        return false;
    }

    validateSaferpayFieldsToken() {
        if (!document.querySelector('#token') || document.querySelector('#token').value == -1) {
            let errorDiv = document.querySelector('#credit-card-error-div');
            errorDiv.innerHTML = '<p>' + creditParams.noDetails + '</p>';
            errorDiv.parentElement.parentElement.parentElement.scrollIntoView({ behavior: 'smooth' });
            errorDiv.parentElement.parentElement.parentElement.focus();
            return false;
        }
        return true;
    }

    submitSaferpayFields() {
        SaferpayFields.submit({
            onSuccess: (evt) => {
                var token = evt.token;
                var formData = new FormData(document.getElementById('confirmOrderForm'));
                formData.append('token', token);
                this._client.post('/saferpay/transaction/initialize/iframe', formData, (response) => {
                    this.setIframeContent(response);
                }, 'application/json', true);
            },

            onError: (evt) => {
                if (!document.querySelector('#token')) {
                    var tokenInput = document.createElement('input');
                    tokenInput.setAttribute('class', 'form-control');
                    tokenInput.setAttribute('id', 'token');
                    tokenInput.setAttribute('name', 'token');
                    tokenInput.setAttribute('value', token);
                    tokenInput.setAttribute('type', 'hidden');
                    creditForm.appendChild(tokenInput);
                } else if (document.querySelector('#token') && (document.querySelector('#token').value == '1' || document.querySelector('#token').value == '-1')) {
                    document.querySelector('#token').setAttribute('value', token);
                    if (document.querySelector('#token').value == '1')
                        document.querySelector(".credit-card-error-div").style = "display:none;";
                }
                return false;
            }

        });
    }
}
