<?php

declare(strict_types=1);

/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware;

use Shopware\Core\Framework\Context;
use Saferpay\Shopware\Installer\CustomFieldsInstaller;
use Saferpay\Shopware\Installer\PaymentMethodInstaller;
use Shopware\Core\Content\Media\File\FileSaver;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;
use Doctrine\DBAL\Connection;

class SaferpayShopware extends Plugin
{
    public const SAFERPAY_PAYMENT_METHOD_ALIPAY = 'ALIPAY';
    public const SAFERPAY_PAYMENT_METHOD_ALIPAY_ID = '6e8207d002324b4a818fccf5da9b7778';
    public const SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS = 'AMEX';
    public const SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID = '787d96a8550c41b4a2d1bc0f02b78b2a';
    public const SAFERPAY_PAYMENT_METHOD_BANCONTACT = 'BANCONTACT';
    public const SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID = '03acac3ea8a14579a98ec54d639e1aff';
    public const SAFERPAY_PAYMENT_METHOD_DINERS_CLUB = 'DINERS';
    public const SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID = 'dcab8a24b3984ebfa598c74329074d85';
    public const SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT = 'DIRECTDEBIT';
    public const SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID = '8dd0273a29a14097a1c6ef2d2d23cd28';
    public const SAFERPAY_PAYMENT_METHOD_E_PRZELEWY = 'EPRZELEWY';
    public const SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID = 'cde28dc641ea4eec92c47ea3badeb0fb';
    public const SAFERPAY_PAYMENT_METHOD_EPS = 'EPS';
    public const SAFERPAY_PAYMENT_METHOD_EPS_ID = '38b1348132e14fe0a179a96bc13cd392';
    public const SAFERPAY_PAYMENT_METHOD_GIROPAY = 'GIROPAY';
    public const SAFERPAY_PAYMENT_METHOD_IDEAL = 'IDEAL';
    public const SAFERPAY_PAYMENT_METHOD_IDEAL_ID = 'ec8004c323c24809bea207eb69472ce0';
    public const SAFERPAY_PAYMENT_METHOD_JCB = 'JCB';
    public const SAFERPAY_PAYMENT_METHOD_JCB_ID = 'dcdc9e1943e048fdb6e7c4a98f714790';
    public const SAFERPAY_PAYMENT_METHOD_KLARNA = 'KLARNA';
    public const SAFERPAY_PAYMENT_METHOD_KLARNA_ID = '3e75c6b884154e808861ee1d07e27a2d';
    public const SAFERPAY_PAYMENT_METHOD_MAESTRO = 'MAESTRO';
    public const SAFERPAY_PAYMENT_METHOD_MAESTRO_ID = '80ecd259d60047fa8dd00c72578ea439';
    public const SAFERPAY_PAYMENT_METHOD_MASTERCARD = 'MASTERCARD';
    public const SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID = '23ce62f6dc1440719fdd43fa15e15dae';
    public const SAFERPAY_PAYMENT_METHOD_MY_ONE = 'MYONE';
    public const SAFERPAY_PAYMENT_METHOD_MY_ONE_ID = 'f26c507f033d4c1eab97e9e7db526826';
    public const SAFERPAY_PAYMENT_METHOD_PAY_PAL = 'PAYPAL';
    public const SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID = '4d60251b642547f09ea3ddbd9df2e089';
    public const SAFERPAY_PAYMENT_METHOD_PAYDIREKT = 'PAYDIREKT';
    public const SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID = 'bfefd77c4a8b4b9f9f0f0a12e2b58129';
    public const SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD = 'POSTCARD';
    public const SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID = 'bd38da578b394cd49dafbbc77b6b25a1';
    public const SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE = 'POSTFINANCE';
    public const SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID = '718a540334594597a9d8d19c0c16de14';
    public const SAFERPAY_PAYMENT_METHOD_SOFORT = 'SOFORT';
    public const SAFERPAY_PAYMENT_METHOD_SOFORT_ID = 'ab6d0be1102d4b29b5c5875f777fa49e';
    public const SAFERPAY_PAYMENT_METHOD_TWINT = 'TWINT';
    public const SAFERPAY_PAYMENT_METHOD_TWINT_ID = 'b73ce9c23d1146a68a0eced750d58ae7';
    public const SAFERPAY_PAYMENT_METHOD_UNIONPAY = 'UNIONPAY';
    public const SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID = '1140f92eac384046a64b606db1e46bde';
    public const SAFERPAY_PAYMENT_METHOD_VISA = 'VISA';
    public const SAFERPAY_PAYMENT_METHOD_VISA_ID = '33446808dce041b9879c15dc1f7f0685';
    public const SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS = 'WLCRYPTOPAYMENTS';
    public const SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID = '31f2338f2cfc447880aa9edf3c72ae85';
    public const SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID = 'bee34f94495349c6ace7564c5483fa67';

    public const SAFERPAY_WALLET_MASTER_PASS = 'MASTERPASS';
    public const SAFERPAY_WALLET_APPLE_PAY = 'APPLEPAY';
    public const SAFERPAY_WALLET_APPLE_PAY_ID = '967b2153e6db4116ad780c4a1e785be4';
    public const SAFERPAY_WALLET_GOOGLE_PAY = 'GOOGLEPAY';
    public const SAFERPAY_WALLET_GOOGLE_PAY_ID = '45dd0ac779154cf7913726f13501ff17';

    public const CUSTOM_FIELD_SAFERPAY_TRANSACTION_ID = 'saferpay_shopware_transaction_id';
    public const CUSTOM_FIELD_SAFERPAY_CAPTURE_ID = 'saferpay_shopware_capture_id';
    public const CUSTOM_FIELD_SAFERPAY_TOKEN = 'saferpay_shopware_token';
    public const CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN = 'saferpay_shopware_notify_token';
    public const THREEDS_FORCE = 'FORCE';

    /**
     * Actions to be performed while installing the plugin
     */
    public function install(InstallContext $installContext): void
    {
        parent::install($installContext);
        $this->getCustomFieldsInstaller($installContext->getContext())->install($installContext);
        $this->getPaymentMethodInstaller($installContext->getContext())->install($installContext);
    }

    /**
     * Creates and returns object of CustomFieldsInstaller
     */
    private function getCustomFieldsInstaller(Context $context): CustomFieldsInstaller
    {
        /** @var EntityRepository $customFieldSetRepository */
        $customFieldSetRepository = $this->container->get('custom_field_set.repository');
        /** @var EntityRepository $customFieldRepository */
        $customFieldRepository = $this->container->get('custom_field.repository');
        $countries = $this->getAllCountries($context);
        return new CustomFieldsInstaller($customFieldSetRepository, $customFieldRepository, $countries);
    }

    /**
     * Get a list of all countries
     */
    private function getAllCountries(Context $context): array
    {
        /** @var EntityRepository $countryTranslationRepository */
        $countryTranslationRepository = $this->container->get('country_translation.repository');
        $countryCriteria = new Criteria();
        $countryCriteria->addAssociation('language');
        $countryCriteria->getAssociation('language')->addAssociation('locale');
        $countryCriteria->addAssociation('country');
        $countryCriteria->getAssociation('country')->addFilter(new EqualsFilter('active', true));
        $countriesList = $countryTranslationRepository->search($countryCriteria, $context)->getEntities();
        $countries = array();
        if (!empty($countriesList)) {
            foreach ($countriesList as $eachCountry) {
                $countries[$eachCountry->getCountry()->getIso()][$eachCountry->getLanguage()->getLocale()->getCode()] = $eachCountry->getName();
            }
        }
        return $countries;
    }

    /**
     * Creates and returns object of PaymentMethodInstaller
     */
    private function getPaymentMethodInstaller(Context $context): PaymentMethodInstaller
    {
        /** @var EntityRepository $paymentRepository */
        $paymentRepository = $this->container->get('payment_method.repository');
        $mediaRepository = $this->container->get('media.repository');
        $mediaFolderRepository = $this->container->get('media_folder.repository');
        $languageRepository = $this->container->get('language.repository');
        $fileSaver = $this->container->get(FileSaver::class);
        return new PaymentMethodInstaller(
            $this->getPluginId($context),
            $paymentRepository,
            $mediaRepository,
            $mediaFolderRepository,
            $languageRepository,
            $fileSaver
        );
    }

    /**
     * Gets the plugin id from pluginIdProvider
     */
    private function getPluginId(Context $context): string
    {
        /** @var PluginIdProvider $pluginIdProvider */
        $pluginIdProvider = $this->container->get(PluginIdProvider::class);
        return $pluginIdProvider->getPluginIdByBaseClass(get_class($this), $context);
    }

    /**
     * Actions to be performed while activating the plugin
     */
    public function activate(ActivateContext $activateContext): void
    {
        parent::activate($activateContext);
        $this->getCustomFieldsInstaller($activateContext->getContext())->activate($activateContext);
    }

    /**
     * Actions to be performed when the plugin is deactivated
     */
    public function deactivate(DeactivateContext $deactivateContext): void
    {
        parent::deactivate($deactivateContext);
        $this->getCustomFieldsInstaller($deactivateContext->getContext())->deactivate($deactivateContext);
        $this->getPaymentMethodInstaller($deactivateContext->getContext())->deactivate($deactivateContext);
    }

    /**
     * Actions to be performed when the plugin is uninstalled
     */
    public function uninstall(UninstallContext $uninstallContext): void
    {
        parent::uninstall($uninstallContext);
        $this->getCustomFieldsInstaller($uninstallContext->getContext())->uninstall($uninstallContext);
        $this->getPaymentMethodInstaller($uninstallContext->getContext())->uninstall($uninstallContext);
        if (!$uninstallContext->keepUserData()) {
            $connectionService = $this->container->get('Doctrine\DBAL\Connection');

            $sqlToDropTransactionTable = <<<SQL
                    DROP TABLE `saferpay_payment_transaction`;
                    SQL;
            $sqlToDropTransactionHistoryTable = <<<SQL
                    DROP TABLE `saferpay_transaction_history`;
                    SQL;
            $sqlToDropCustomerSecureTranactionTable = <<<SQL
                    DROP TABLE `saferpay_customer_secure_transaction`;
                    SQL;
            $connectionService->executeStatement($sqlToDropTransactionHistoryTable);
            $connectionService->executeStatement($sqlToDropTransactionTable);
            $connectionService->executeStatement($sqlToDropCustomerSecureTranactionTable);
        }
    }
}
