<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Service;

use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Psr\Log\LoggerInterface;
use Stringable;
use Symfony\Component\HttpKernel\KernelInterface;

class LoggerService implements LoggerInterface
{
    private $logger;
    private $kernel;
    const SAFERPAY_LOG_PREFIX = 'Saferpay : ';

    public function __construct(KernelInterface $kernel)
    {
        $this->kernel = $kernel;
        $this->logger = new Logger('Saferpay_Shopware');
        $logFilePath =  $this->kernel->getProjectDir() . '/var/log/saferpay-shopware/' . date('Y-m-d') . '.log';
        $streamHandler = new StreamHandler($logFilePath);
        $this->logger->pushHandler($streamHandler);
    }

    public function emergency(string|Stringable $message, array $context = []): void
    {
        $this->logger->emergency(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function debug(string|Stringable $message, array $context = []): void
    {
        $this->logger->debug(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function log($level, string|Stringable $message, array $context = []): void
    {
        $this->logger->log($level, self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function error(string|Stringable $message, array $context = []): void
    {
        $this->logger->error(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function warning(string|Stringable $message, array $context = []): void
    {
        $this->logger->warning(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function alert(string|Stringable $message, array $context = []): void
    {
        $this->logger->alert(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function critical(string|Stringable $message, array $context = []): void
    {
        $this->logger->critical(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function info(string|Stringable $message, array $context = []): void
    {
        $this->logger->info(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public function notice(string|Stringable $message, array $context = []): void
    {
        $this->logger->notice(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }

    public static function logRequestOrResponse(string|Stringable $message, string $type, array $context = []): void
    {
        // Determine logger name and log file path based on request or response type
        $logName = $type == 'request' ? 'Saferpay_Shopware_Request' : 'Saferpay_Shopware_Response';
        $logFilePath = __DIR__ . '/../../../../var/log/saferpay-shopware/' . date('Y-m-d') . '.log';

        // Create a new logger instance with the appropriate name
        $logger = new Logger($logName);

        // Set up the stream handler with the log file path
        $streamHandler = new StreamHandler($logFilePath);
        $logger->pushHandler($streamHandler);

        // Log the message with the context
        $logger->info(self::SAFERPAY_LOG_PREFIX . $message, $context);
    }
}
