<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Storefront\Controller;

use Symfony\Component\HttpFoundation\RedirectResponse;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayTransactionHistoryEntity;
use Saferpay\Shopware\Service\LoggerService;
use Shopware\Core\Checkout\Order\SalesChannel\OrderService;
use Saferpay\Shopware\Service\PaymentService;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\Aggregate\SalesChannelDomain\SalesChannelDomainEntity;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Controller\StorefrontController;
use Shopware\Storefront\Page\GenericPageLoader;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Shopware\Core\Framework\Log\Package;

#[Route(defaults: ['_routeScope' => ['storefront']])]
#[Package('storefront')]
class SaferpayIframeController extends StorefrontController
{


    private $orderService;
    private $paymentService;
    private $logger;
    private $pageLoader;

    /**
     * Initialize services used in controller
     */
    public function __construct(
        OrderService $orderService,
        PaymentService $paymentService,
        LoggerService $logger,
        GenericPageLoader $genericPageLoader
    ) {
        $this->orderService   = $orderService;
        $this->paymentService = $paymentService;
        $this->logger = $logger;
        $this->pageLoader = $genericPageLoader;
    }
    
    
    #[Route(path: '/saferpay/transaction/initialize/iframe', name: 'frontend.saferpay.transaction.initialize', defaults:['XmlHttpRequest' => true], methods: ['POST'])]
    public function initialize(RequestDataBag $requestData, SalesChannelContext $context): Response
    {
        $parameters     = $requestData->get('parameters');
        $requestStack   = $this->container->get('request_stack');
        if (empty($parameters)) {
            $parameters = $_POST;
        }
        $currentOrderId = $requestStack->getSession()->get('currentOrderId');
        $langId = $requestStack->getSession()->get('currentLangId');
        if (!$currentOrderId) {
            $orderId = $this->orderService->createOrder($requestData, $context);
        } else {
            $orderId = $currentOrderId;
        }
        [$order, $orderTransaction] = $this->getOrderAndTransactionFromOrderId($orderId, $context);
        $this->saveOrderCustomFields($parameters, $order, $context->getContext());
        if (isset($parameters['customer_selected_alias'])) {
            $order->setCustomFields(['customer_selected_alias' => $parameters['customer_selected_alias']]);
        }
        if (isset($parameters['token'])) {
            $order->setCustomFields(['saferpayFieldsToken' => $parameters['token']]);
        }
        
        if ($langId === '' || $langId === null) {
            $langId = $context->getLanguageId();
        }
        if ($langId !== "") {
            $orderRepository  = $this->container->get('order.repository');
            $orderRepository->update([[
                   'id' => $order->getId(),
                   'languageId' => $langId
            ]], $context->getContext());
            $order->setLanguageId($langId);
        }
        if (!empty($parameters) && isset($parameters['paymentMethodId'])) {
            $orderTransaction->setPaymentMethodId($parameters['paymentMethodId']);
            $orderTransactionRepository = $this->container->get('order_transaction.repository');
            $updateData                 = array(
                   'id'              => $orderTransaction->getId(),
                   'paymentMethodId' => $parameters['paymentMethodId'],
            );
            $orderTransactionRepository->update(array($updateData), $context->getContext());
        } else {
            $orderTransaction->setPaymentMethod($context->getPaymentMethod());
            $orderTransaction->setPaymentMethodId($context->getPaymentMethod()->getId());
            $orderTransactionRepository = $this->container->get('order_transaction.repository');
            $updateData                 = array(
                   'id'              => $orderTransaction->getId(),
                   'paymentMethodId' => $context->getPaymentMethod()->getId(),
            );
            $orderTransactionRepository->update(array($updateData), $context->getContext());
        }
        $router            = $this->container->get('router');
        $currentRequestUrl = $router->generate(
            'frontend.saferpay.transaction.authorize',
            array('orderId' => $orderId),
            $router::ABSOLUTE_URL
        );
        $redirectUrl       = $this->paymentService->initializePayment(
            $orderTransaction,
            $order,
            $currentRequestUrl,
            $context->getSalesChannel(),
            $context->getContext(),
            $langId
        );
        
        $noRedirect = false;
        if (!$order->getCustomFieldsValue('saferpayFieldsToken') && $redirectUrl === $currentRequestUrl) {
            $noRedirect = true;
            $this->redirect($redirectUrl, 302);
        }
        return new JsonResponse(
            array(
                      'iframeUrl' => $redirectUrl,
                      'orderId'   => $orderId,
                      'noRedirect' => $noRedirect
               )
        );
    }

    /**
     * Save custom fields to order
     */
    private function saveOrderCustomFields(array $formData, OrderEntity $order, Context $context)
    {
        $orderRepository  = $this->container->get('order.repository');
        $customFields = $order->getCustomFields();
        if (
            !empty($formData) && isset($formData['save_card_to_account'])
            && $formData['save_card_to_account'] === 'on'
        ) {
            $customFields['save_card_to_account'] = 'on';
        } else {
            if (isset($customFields['save_card_to_account'])) {
                unset($customFields['save_card_to_account']);
            }
        }
        if (
            !empty($formData) && isset($formData['customer_selected_alias'])
            && $formData['customer_selected_alias'] !== ''
        ) {
            $customFields['customer_selected_alias'] = $formData['customer_selected_alias'];
        } else {
            if (isset($customFields['customer_selected_alias'])) {
                unset($customFields['customer_selected_alias']);
            }
        }
        if (
            !empty($formData) && isset($formData['token'])
            && $formData['token'] !== ''
        ) {
            $customFields['saferpayFieldsToken'] = $formData['token'];
        } else {
            if (isset($customFields['saferpayFieldsToken'])) {
                unset($customFields['saferpayFieldsToken']);
            }
        }
        $updateData = [
               'id' => $order->getId(),
               'customFields' => $customFields
        ];
        $orderRepository->update([$updateData], $context);
    }

    #[Route(path: '/saferpay/transaction/authorize/iframe/{orderId}', name: 'frontend.saferpay.transaction.authorize', defaults:['XmlHttpRequest' => true], methods: ['GET'])]
    public function authorize(string $orderId, Request $request, SalesChannelContext $context)
    {
        [$order, $orderTransaction] = $this->getOrderAndTransactionFromOrderId($orderId, $context);
        $requestStack   = $this->container->get('request_stack');
        $langId = $requestStack->getSession()->get('currentLangId');

        if (!$langId || $langId === "") {
            $requestStack->getSession()->set('currentLangId', $order->getLanguageId());
        }
       
        try {
            $this->paymentService->finalizePayment(
                $orderTransaction,
                $context->getSalesChannel(),
                $context->getContext(),
                'iframe'
            );
            $page             = $this->pageLoader->load($request, $context);
            return $this->renderStorefront(
                '@SaferpayShopware/storefront/page/saferpay-shopware/saved-card/loading.html.twig',
                array(
                          'page' => $page
                   )
            );
        } catch (\Throwable $th) {
            $errorUrl       = $this->generateUrl(
                'frontend.account.edit-order.page',
                array(
                          'orderId'    => $order->getId(),
                          'error-code' => 'CHECKOUT__ASYNC_PAYMENT_FINALIZE_INTERRUPTED'
                   ),
                UrlGeneratorInterface::ABSOLUTE_PATH
            );
            return new RedirectResponse($errorUrl);
        }
    }

    /**
     * Get order and order transaction objects from order id
     *
     * @param string $orderId order id.
     * @param object $context context to use in database querying.
     */
    private function getOrderAndTransactionFromOrderId($orderId, $context)
    {
        $orderTransactionRepository = $this->container->get('order_transaction.repository');
        $criteria                   = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $orderId))
               ->addAssociations(array('order, order_version, state, payment_method'));
        /**
         * @var OrderTransactionEntity
         */
        $orderTransaction = $orderTransactionRepository->search($criteria, $context->getContext())->last();
        $orderRepository  = $this->container->get('order.repository');
        $orderCriteria    = new Criteria([$orderId]);
        $orderCriteria->addAssociation('currency');
        $orderCriteria->addAssociation('language');
        $orderCriteria->addAssociation('saleschannel');
        $orderCriteria->addAssociation('billingAddress');
        $orderCriteria->addAssociation('deliveries.shippingOrderAddress.salutation');
        $orderCriteria->addAssociation('deliveries.shippingOrderAddress.country');
        $orderCriteria->addAssociation('deliveries.shippingOrderAddress.countryState');
        $orderCriteria->addAssociation('billingAddress.country');
        $order = $orderRepository->search($orderCriteria, $context->getContext())->last();
        return array($order, $orderTransaction);
    }

    #[Route(path: '/saferpay/transaction/order/status/{orderId}', name: 'frontend.saferpay.transaction.order.status', defaults:['XmlHttpRequest' => true], methods: ['GET'])]
    public function checkOrderStatus(string $orderId, Request $request, SalesChannelContext $context): Response
    {
        [$order, $orderTransaction] = $this->getOrderAndTransactionFromOrderId($orderId, $context);
        $langId = $order->getLanguageId();
        $domainCriteria = new Criteria();
        $domainCriteria->addFilter(new EqualsFilter('salesChannelId', $context->getSalesChannelId()))
               ->addFilter(new EqualsFilter('languageId', $langId));

        /**
         * @var SalesChannelDomainEntity
         */
        $salesChannelDomain = $this->container->get('sales_channel_domain.repository')
               ->search($domainCriteria, $context->getContext())->first();

        $context->getLanguageId();
        $paymentTransaction         = $this->paymentService->paymentTransactionSaveService
               ->getPaymentTransactionFromOrderTransactionId($orderTransaction->getId(), $context->getContext());
        $isCancel                   = false;
        $paymentHistoryRepository   = $this->container->get('saferpay_transaction_history.repository');
        // If no status or only initialized, do not redirect
        $cancelCriteria = new Criteria();
        $cancelCriteria->addFilter(new EqualsFilter('paymentTransactionId', $paymentTransaction->getId()));
        /** @var SaferpayTransactionHistoryEntity */
        $cancelHistory = $paymentHistoryRepository->search($cancelCriteria, $context->getContext())->last();
        if (
            $cancelHistory &&
            ($cancelHistory->getTransactionStatus() === $this->paymentService
                   ->paymentTransactionSaveService::API_PAYMENT_STATUS_CANCEL
                   || $cancelHistory->getTransactionStatus() ===
                   $this->paymentService->paymentTransactionSaveService::API_PAYMENT_STATUS_FAILED)
        ) {
            $isCancel = true;
        }
        // redirect if authorized
        $authorizeCriteria = new Criteria();
        $authorizeCriteria->addFilter(new EqualsFilter('paymentTransactionId', $paymentTransaction->getId()));
        $authorizeCriteria->addFilter(
            new EqualsFilter(
                'transactionStatus',
                $this->paymentService->paymentTransactionSaveService::API_PAYMENT_STATUS_AUTHORIZED
            )
        );
        $authorizeHistory = $paymentHistoryRepository->search(
            $authorizeCriteria,
            $context->getContext()
        )->first();
        $redirect         = false;
        $redirectStatus   = 'no-redirect';
        $errorUrl         = '';
        if ($authorizeHistory || $isCancel) {
            $redirect = true;
        }
        if ($authorizeHistory && $isCancel) {
            $redirectStatus = 'cancel';
            $errorUrl       = $this->generateUrl(
                'frontend.account.edit-order.page',
                array(
                          'orderId'    => $order->getId(),
                          'error-code' => 'CHECKOUT__ASYNC_PAYMENT_FINALIZE_INTERRUPTED'
                   ),
                UrlGeneratorInterface::ABSOLUTE_PATH
            );
        } elseif ($authorizeHistory) {
            $redirectStatus = 'authorize';
        } elseif ($isCancel) {
            $redirectStatus = 'cancel';
            if (
                $cancelHistory->getTransactionStatus() ===
                $this->paymentService
                       ->paymentTransactionSaveService::API_PAYMENT_STATUS_CANCEL
            ) {
                $errorCode = 'CHECKOUT__CUSTOMER_CANCELED_EXTERNAL_PAYMENT';
            } else {
                $errorCode = 'CHECKOUT__ASYNC_PAYMENT_FINALIZE_INTERRUPTED';
            }
            $errorUrl       = $this->generateUrl(
                'frontend.account.edit-order.page',
                array(
                          'orderId'    => $order->getId(),
                          'error-code' => $errorCode
                   ),
                UrlGeneratorInterface::ABSOLUTE_PATH
            );
        }
        $orderTransactionRepository = $this->container->get('order_transaction.repository');
        $criteria = new Criteria([$orderTransaction->getId()]);
        $criteria->addAssociation('stateMachineState');
        $orderTransaction = $orderTransactionRepository->search($criteria, $context->getContext())->first();
        $paymentState = $orderTransaction->getStateMachineState()->getName();
        if ($paymentState === 'Cancelled') {
            $errorUrl = $this->generateUrl(
                'frontend.account.edit-order.page',
                array(
                          'orderId'    => $order->getId(),
                          'error-code' => 'CHECKOUT__CUSTOMER_CANCELED_EXTERNAL_PAYMENT'
                   ),
                UrlGeneratorInterface::ABSOLUTE_PATH
            );
        }
        return new JsonResponse(
            array(
                      'redirectTo'     => $redirect,
                      'redirectStatus' => $redirectStatus,
                      'paymentState'   => $paymentState,
                      'errorUrl'       => $salesChannelDomain->getUrl() . $errorUrl,
               )
        );
    }
}
