<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Storefront\Controller;

use DateTime;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Sorting\FieldSorting;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Doctrine\DBAL\Exception as DBALException;
use Exception;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayCustomerSecureTransactionEntity;
use Saferpay\Shopware\SaferpayShopware;
use Saferpay\Shopware\Service\LoggerService;
use Saferpay\Shopware\Service\PaymentService;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Storefront\Page\GenericPageLoader;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\OptionsResolver\Exception\MissingOptionsException;
use Symfony\Component\Validator\Exception\ValidatorException;
use Shopware\Core\Framework\Log\Package;

#[Route(defaults: ['_routeScope' => ['storefront']])]
#[Package('storefront')]
class SaferpaySaveCardController extends StorefrontController
{

    public const LOGIN_REQUIRED = "_loginRequired";
    public GenericPageLoader $pageloader;
    public PaymentService $paymentService;
    public LoggerService $logger;

    /**
     * Initialize the constructor variables
     */
    public function __construct(
        GenericPageLoader $genericPageLoader,
        PaymentService $paymentService,
        LoggerService $logger
    ) {
        $this->pageloader     = $genericPageLoader;
        $this->paymentService = $paymentService;
        $this->logger         = $logger;
    }

    #[Route(path: '/saferpay/account/saved/cards', name: 'frontend.account.saferpay.saved.cards', defaults: [self::LOGIN_REQUIRED => true], methods: ['GET'])]
    public function displaySavedCards(Request $request, SalesChannelContext $context): Response
    {
        $currentUser = $context->getCustomerId();
        $criteria    = new Criteria();
        $criteria->addFilter(new EqualsFilter('customerId', $currentUser));
        $criteria->addFilter(new EqualsFilter('aliasActive', 'Y'));
        $criteria->addSorting(new FieldSorting('id', 'DESC'));
        $savedCards = $this->container->get('saferpay_customer_secure_transaction.repository')
            ->search($criteria, $context->getContext())->getEntities();
        $session    = $request->getSession();
        $cardType   = $session->get('cardType');
        if (!$cardType) {
            $cardType = 'cards';
        }
        $creditParams = array();
        if ($cardType == 'cards') {
            $cardCriteria = new Criteria();
            $cardCriteria->addFilter(new EqualsFilter('id', SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID));
            /**
             * @var PaymentMethodEntity
             */
            $creditCardPayment      = $this->container->get('payment_method.repository')
                ->search($cardCriteria, $context->getContext())->first();
            $creditCardCustomFields = $creditCardPayment->getCustomFields();
            $saveCardTheme          = $this->paymentService->paymentTransactionSaveService
                ->getPluginConfig('saveCardStyle', $context->getSalesChannel());
            $translator             = $this->container->get('translator');
            $creditParams           = array(
                'accessToken'                            => $creditCardCustomFields['saferpay_shopware_field_access_token'] ?? '',
                'sourceUrl'                              => $creditCardCustomFields['saferpay_shopware_field_api_url'] ?? '',
                'cardHolderName'                         => $translator->trans('cardTheme.cardholders'),
                'expiration'                             => 'MM/YY',
                'cssUrl'                                 => $creditCardCustomFields['saferpay_shopware_field_css_url'] ?? '',
                'errorMessage'                           => $translator->trans('cardTheme.errorMessage'),
                'cardTemplate'                           => $saveCardTheme,
                'paymentMethods'                         => $creditCardCustomFields['saferpay_shopware_payment_means'] ?? '',
                'labels'                                 => array(
                    'invalid'     => $translator->trans('cardTheme.invalidLabel'),
                    'empty'       => $translator->trans('cardTheme.emptyLabel'),
                    'unsupported' => $translator->trans('cardTheme.unsupportedLabel'),
                    'expired'     => $translator->trans('cardTheme.expiredLabel'),
                ),
                'noDetails'                              => $translator->trans('cardTheme.noDetails'),
                'saferpay_shopware_field_js_library_url' =>
                $creditCardCustomFields['saferpay_shopware_field_js_library_url'] ?? '',
            );
        }
        $alreadySavedCard = $this->unsetCustomFieldAfterUse('alreadySavedCard', $context);
        $cardSaved        = $this->unsetCustomFieldAfterUse('cardSaved', $context);
        $cardSaveError    = $this->unsetCustomFieldAfterUse('cardSaveError', $context);
        $page             = $this->pageloader->load($request, $context);
        $cardDeleted = $request->query->get('cardDeleted', false);
        $cardUpdated = $request->query->get('cardUpdated', false);
        return $this->renderStorefront(
            '@SaferpayShopware/storefront/page/saferpay-shopware/saved-card/display-saved-cards.html.twig',
            array(
                'savedCards'       => $savedCards,
                'cardType'         => $cardType ?? 'cards',
                'creditParams'     => $creditParams,
                'alreadySavedCard' => $alreadySavedCard,
                'cardSaved'        => $cardSaved,
                'cardSaveError'    => $cardSaveError,
                'page'             => $page,
                'cardDeleted'      => $cardDeleted,
                'cardUpdated'      => $cardUpdated
            )
        );
    }

    /**
     * Update customer custom field to false after use
     */
    private function unsetCustomFieldAfterUse($customField, $context)
    {
        $customFieldValue = $context->getCustomer()->getCustomFieldsValue($customField);
        if ($customFieldValue) {
            $customerRepository = $this->container->get('customer.repository');
            $customer           = $context->getCustomer();
            $customer->setCustomFields(array($customField => false));
            $updateCustomerData = array(
                'id'           => $customer->getId(),
                'customFields' => $customer->getCustomFields(),
            );
            $customerRepository->update(array($updateCustomerData), $context->getContext());
        }
        return $customFieldValue;
    }

    #[Route(path: '/saferpay/account/saved/cards/edit/{cardId}', name: 'frontend.account.saferpay.saved.card.edit', defaults:['XmlHttpRequest' => true, self::LOGIN_REQUIRED => true], methods: ['GET'])]
    public function editCard(Request $request, SalesChannelContext $context, string $cardId): JsonResponse
    {
        if (!$cardId || $cardId === '') {
            $this->logger->error(
                'Error Code : 1030 : No card id to process updation of card'
            );
            throw new MissingOptionsException('No card id to process updation of card');
        }
        $customerSecureTransactionRepository = $this->container
            ->get('saferpay_customer_secure_transaction.repository');
        /**
         * @var SaferpayCustomerSecureTransactionEntity
         */
        $savedCard = $customerSecureTransactionRepository->search(array($cardId), $context->getContext())->first();
        $expMonth  = '';
        $expYear   = '';
        if ($savedCard) {
            $expMonth = $savedCard->getSaferpayExpMonth();
            $expYear  = $savedCard->getSaferpayExpYear();
        }
        return new JsonResponse(
            array(
                'expMonth' => $expMonth,
                'expYear'  => $expYear,
            )
        );
    }

    #[Route(path: '/saferpay/account/saved/cards/submit/edit', name: 'frontend.account.saferpay.saved.card.edit.submit', defaults:['XmlHttpRequest' => true, self::LOGIN_REQUIRED => true], methods: ['POST'])]
    public function editCardSubmit(Request $request, SalesChannelContext $context)
    {
        $monthSelected                       = $request->request->get('month');
        $yearSelected                        = $request->request->get('year');
        $payloadData                         = array(
            'expMonth' => $monthSelected,
            'expYear'  => $yearSelected,
        );
        $cardId                              = $request->request->get('cardId');
        $customerSecureTransactionRepository = $this->container
            ->get('saferpay_customer_secure_transaction.repository');
        $customerSecureTransaction           = $customerSecureTransactionRepository
            ->search(new Criteria(array($cardId)), $context->getContext())->first();
        if ($customerSecureTransaction) {
            $payloadData['aliasId'] = $customerSecureTransaction->getSaferpayAliasId();
            $paymentApiClient       = $this->paymentService
                ->createSaferpayPaymentApiClientForSalesChannel($context->getSalesChannel());
            try {
                $response = $paymentApiClient->aliasUpdate($payloadData);
            } catch (Exception $ex) {
                $this->logger->error(
                    'Error Code : 1028 : Error occured while sending update saved card API ' . $ex->getMessage()
                );
            }
            $updateCardData = array(
                'id'                    => $customerSecureTransaction->getId(),
                'saferpayAliasLifetime' => $response->Alias->Lifetime,
                'saferpayAliasId'       => $response->Alias->Id,
                'saferpayExpMonth'      => isset($response->PaymentMeans->Card->ExpMonth) ? (string) $response->PaymentMeans->Card->ExpMonth : '',
                'saferpayExpYear'       => isset($response->PaymentMeans->Card->ExpYear) ? (string) $response->PaymentMeans->Card->ExpYear : '',
            );
            try {
                $customerSecureTransactionRepository->update(array($updateCardData), $context->getContext());
            } catch (Exception $ex) {
                $this->logger->error(
                    'Error Code : 1029 : Error occured while updation of saved card in db ' . $ex->getMessage()
                );
            }
        } else {
            $this->logger->error('Error Code : 1026 : Error occured while fetching saved card for updation');
        }
        return $this->redirectToRoute('frontend.account.saferpay.saved.cards', ['cardUpdated' => 'true']);
    }

    #[Route(path: '/saferpay/account/saved/cards/delete/{cardId}', name: 'frontend.account.saferpay.saved.card.delete', defaults:['XmlHttpRequest' => true, self::LOGIN_REQUIRED => true], methods: ['GET'])]
    public function deleteCard(Request $request, SalesChannelContext $context, string $cardId): JsonResponse
    {
        $status = '';
        if (!$cardId || $cardId === '') {
            $this->logger->error(
                'Error Code : 1030 : No card id to process deletion of card'
            );
            throw new MissingOptionsException('No card id to process deletion of card');
        }
        try {
            $paymentApiClient                    = $this->paymentService
                ->createSaferpayPaymentApiClientForSalesChannel($context->getSalesChannel());
            $customerSecureTransactionRepository = $this->container
                ->get('saferpay_customer_secure_transaction.repository');
            /**
             * @var SaferpayCustomerSecureTransactionEntity
             */
            $customerSecureTransaction = $customerSecureTransactionRepository
                ->search(new Criteria(array($cardId)), $context->getContext())->first();
            if ($customerSecureTransaction) {
                try {
                    $paymentApiClient->aliasDelete($customerSecureTransaction->getSaferpayAliasId());
                } catch (Exception $ex) {
                    $this->logger->error(
                        'Error Code : 1028 : Error occured while sending delete saved card API ' . $ex->getMessage()
                    );
                }
                try {
                    $customerSecureTransactionRepository->delete(array(array('id' => $cardId)), $context->getContext());
                } catch (Exception $ex) {
                    $this->logger->error(
                        'Error Code : 1029 : Error occured while deletion of saved card in db ' . $ex->getMessage()
                    );
                }
                $status = 'success';
            } else {
                $this->logger->error('Error Code : 1026 : Error occured while fetching saved card for deletion');
            }
        } catch (Exception $ex) {
            $status = 'failure';
            $this->logger->error('Error Code : 1034 : Error occured while deleting saved card ' . $ex->getMessage());
            throw new DBALException('Error occured while deleting saved card - ' . $ex->getMessage());
        }

        return new JsonResponse(
            array(
                'status' => $status,
            )
        );
    }

    #[Route(path: '/saferpay/account/saved/card/change/card/type', name: 'frontend.account.saferpay.saved.card.change.add.card.type', defaults:['XmlHttpRequest' => true, self::LOGIN_REQUIRED => true], methods: ['POST'])]
    public function changeCardType(Request $request, SalesChannelContext $context)
    {
        $cardType = $request->request->get('cardType');
        $session  = $request->getSession();
        $session->set('cardType', $cardType);
        return $this->redirectToRoute('frontend.account.saferpay.saved.cards');
    }

    #[Route(path: '/saferpay/account/saved/card/add/card', name: 'frontend.account.saferpay.saved.card.add.card', defaults:['XmlHttpRequest' => true, self::LOGIN_REQUIRED => true], methods: ['POST','GET'])]
    public function addCard(Request $request, SalesChannelContext $context)
    {
        $saferpayAccessToken = $request->request->get('saferpayFieldsAccessToken');
        $paymentApiClient     = $this->paymentService
            ->createSaferpayPaymentApiClientForSalesChannel($context->getSalesChannel());
        $payloadData          = array();
        $session              = $request->getSession();
        $cardType             = $session->get('cardType');
        if (!$cardType) {
            $cardType = 'cards';
        }
        $payloadData['type']                      = $cardType === 'cards' ? 'CARD' : strtoupper($cardType);
        $payloadData['saferpayFieldsAccessToken'] = $saferpayAccessToken;
        $operationMode                            = $this->paymentService->paymentTransactionSaveService
            ->getPluginConfig('operationMode', $context->getSalesChannel());
        if ($operationMode === 'test') {
            $payloadData['terminalId'] = $this->paymentService->paymentTransactionSaveService
                ->getPluginConfig('terminalIdTest', $context->getSalesChannel());
        } else {
            $payloadData['terminalId'] = $this->paymentService->paymentTransactionSaveService
                ->getPluginConfig('terminalIdLive', $context->getSalesChannel());
        }
        $payloadData['theme'] = $this->paymentService->paymentTransactionSaveService
            ->getPluginConfig('theme', $context->getSalesChannel());
        $cardCriteria         = new Criteria();
        $cardCriteria->addFilter(new EqualsFilter('id', SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID));
        $creditCardPayment        = $this->container->get('payment_method.repository')
            ->search($cardCriteria, $context->getContext())->first();
        $creditCardCustomFields   = $creditCardPayment->getCustomFields();
        $payloadData['cssUrl']    = $creditCardCustomFields['saferpay_shopware_field_css_url'] ?? '';
        $router                   = $this->container->get('router');
        $returnUrl                = $router->generate(
            'frontend.account.saferpay.saved.card.assert.add.card',
            array('customerId' => $context->getCustomerId()),
            $router::ABSOLUTE_URL
        );
        $payloadData['returnUrl'] = $returnUrl;
        try {
            $response = $paymentApiClient->aliasInsert($payloadData);
        } catch (Exception $ex) {
            $this->logger->error('Error Code : 1006 : No request id in response ' . $ex->getMessage());
            return new JsonResponse(
                array(
                    'iframeUrl'     => '',
                    'redirect'      => 'no',
                    'saferpayToken' => '',
                )
            );
        }
        $customer = $context->getCustomer();
        if (!$customer) {
            $this->logger->error('Error Code : 1033 : No valid customer id to save card');
            throw new ValidatorException('No valid customer id to save card');
        }
        $customer->setCustomFields(array('saferpayToken' => $response->Token));
        $customerRepository = $this->container->get('customer.repository');
        $updateCustomerData = array(
            'id'           => $customer->getId(),
            'customFields' => $customer->getCustomFields(),
        );
        $customerRepository->update(array($updateCustomerData), $context->getContext());
        if ($response->RedirectRequired) {
            $redirectUrl = $response->Redirect->RedirectUrl;
        } else {
            $redirectUrl = $returnUrl;
        }
        return new JsonResponse(
            array(
                'iframeUrl'     => $redirectUrl,
                'redirect'      => $response->RedirectRequired ? 'yes' : 'no',
                'saferpayToken' => $response->Token,
                'cardType'      => $cardType,
            )
        );
    }

    #[Route(path: '/saferpay/account/saved/card/assert/add/card/{customerId}', name: 'frontend.account.saferpay.saved.card.assert.add.card', defaults:['XmlHttpRequest' => true], methods: ['GET'])]
    public function assertAddCard(Request $request, SalesChannelContext $context, string $customerId)
    {
        $customerRepository = $this->container->get('customer.repository');
        $customerCriteria   = new Criteria(array($customerId));
        /**
         * @var CustomerEntity
         */
        $customer = $customerRepository->search($customerCriteria, $context->getContext())->first();
        if ($customer) {
            $saferpayToken = $customer->getCustomFieldsValue('saferpayToken');
        } else {
            $this->logger->error('Error Code : 1033 : No valid customer id to save card');
            throw new ValidatorException('No valid customer id to save card');
        }
        $session  = $request->getSession();
        $cardType = $session->get('cardType');
        if (!$saferpayToken) {
            return $this->returnBasedOnCardType($cardType, 'error', $context, $request);
        }
        $paymentApiClient = $this->paymentService
            ->createSaferpayPaymentApiClientForSalesChannel($context->getSalesChannel());
        try {
            $response = $paymentApiClient->assertAliasInsert($saferpayToken);
        } catch (Exception $ex) {
            $this->logger->error('Error Code : 1006 : No request id in response ' . $ex->getMessage());
            $this->updateCustomerCustomFields($customer, $customerRepository, $context, false);
            return $this->returnBasedOnCardType($cardType, 'error', $context, $request);
        }

        $customerSecureTransactionRepository = $this->container->get('saferpay_customer_secure_transaction.repository');
        /**
         * Check if the alias id already exists
         */
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('saferpayAliasId', $response->Alias->Id));
        $criteria->addFilter(new EqualsFilter('customerId', $customer->getId()));
        $previousSaveOfCard = $customerSecureTransactionRepository->search($criteria, $context->getContext())->first();
        if ($previousSaveOfCard) {
            $customer->setCustomFields(array('alreadySavedCard' => true));
            $updateCustomerData = array(
                'id'           => $customer->getId(),
                'customFields' => $customer->getCustomFields(),
            );
            $customerRepository->update(array($updateCustomerData), $context->getContext());
            $this->logger->error('Error Code : 1027 : Duplicate alias');
            return $this->returnBasedOnCardType($cardType, 'error', $context, $request);
        }
        $operationMode = $this->paymentService->paymentTransactionSaveService
            ->getPluginConfig('operationMode', $context->getSalesChannel());
        if ($operationMode === 'test') {
            $saferpayCustomerId = $this->paymentService->paymentTransactionSaveService
                ->getPluginConfig('customerIdTest', $context->getSalesChannel());
        } else {
            $saferpayCustomerId = $this->paymentService->paymentTransactionSaveService
                ->getPluginConfig('customerIdLive', $context->getSalesChannel());
        }
        $saveCardData = array(
            'saferpayRequestId'     => $response->ResponseHeader->RequestId ?? null,
            'saferpayToken'         => $saferpayToken,
            'saferpayAliasId'       => $response->Alias->Id ?? null,
            'saferpayAliasLifetime' => $response->Alias->Lifetime ?? null,
            'saferpayPaymentMethod' => $response->PaymentMeans->Brand->PaymentMethod ?? null,
            'saferpayDisplayText'   => $this->paymentService->paymentTransactionSaveService
                ->getSaferpayDisplayText($response),
            'saferpayMaskedNumber'  => $this->paymentService->paymentTransactionSaveService->getMaskedNumber($response),
            'saferpayHolderName'    => $this->paymentService->paymentTransactionSaveService->getHolderName($response),
            'customerId'            => $customer->getId(),
            'saferpayCustomerId'    => (int) $saferpayCustomerId,
            'liveTransaction'       => 'N',
            'aliasActive'           => 'Y',
            'saferpayCreatedAt'     => new DateTime('now'),
        );
        if (isset($response->CheckResult->Result)) {
            $saveCardData['saferpayAuthenticated'] =
                ($response->CheckResult->Result == 'OK' || $response->CheckResult->Result == 'OK_AUTHENTICATED')
                ? 'Yes' : 'NA';
        } else {
            $saveCardData['saferpayAuthenticated'] = 'NA';
        }
        if (isset($response->PaymentMeans->Card->ExpYear)) {
            $saveCardData['saferpayExpYear'] = (string) $response->PaymentMeans->Card->ExpYear;
        }
        if (isset($response->PaymentMeans->Card->ExpMonth)) {
            $saveCardData['saferpayExpMonth'] = (string) $response->PaymentMeans->Card->ExpMonth;
        }
        try {
            $customerSecureTransactionRepository->create(array($saveCardData), $context->getContext());
        } catch (Exception $ex) {
            $this->logger->error('Error Code : 1008 : Error while saving card ' . $ex->getMessage());
            throw new DBALException('Error while saving card ' . $ex->getMessage());
        }
        $this->updateCustomerCustomFields($customer, $customerRepository, $context, true);
        return $this->returnBasedOnCardType($cardType, 'success', $context, $request);
    }

    /**
     * Return based on the type of card
     */
    private function returnBasedOnCardType($cardType, $status, $context, $request)
    {
        if ($cardType == 'postfinance') {
            return new RedirectResponse($this->generateUrl('frontend.account.saferpay.saved.cards'));
        } else {
            $page             = $this->pageloader->load($request, $context);
            return $this->renderStorefront(
                '@SaferpayShopware/storefront/page/saferpay-shopware/saved-card/loading.html.twig',
                array(
                    'page' => $page,
                    'status' => $status
                )
            );
        }
    }

    /**
     * Update customer custom fields
     */
    private function updateCustomerCustomFields($customer, $customerRepository, $context, $cardSaved)
    {
        if ($cardSaved) {
            $customer->setCustomFields(
                array(
                    'saferpayToken' => '',
                    'cardSaved'     => $cardSaved,
                )
            );
        } else {
            $customer->setCustomFields(
                array(
                    'saferpayToken' => '',
                    'cardSaveError' => true,
                )
            );
        }

        $updateCustomerData = array(
            'id'           => $customer->getId(),
            'customFields' => $customer->getCustomFields(),
        );
        $customerRepository->update(array($updateCustomerData), $context->getContext());
    }

    #[Route(path: '/saferpay/account/saved/card/check/assert/add/card', name: 'frontend.account.saferpay.saved.card.check.assert.add.card', defaults:['XmlHttpRequest' => true], methods: ['GET'])]
    public function checkAssertAddCard(Request $request, SalesChannelContext $context)
    {
        if ($context->getCustomer()) {
            $saferpayToken    = $context->getCustomer()->getCustomFieldsValue('saferpayToken');
            $alreadySavedCard = $context->getCustomer()->getCustomFieldsValue('alreadySavedCard');
            if (!$saferpayToken || $saferpayToken == '' || $alreadySavedCard) {
                return new JsonResponse(
                    array(
                        'action' => 'complete',
                    )
                );
            }
        }
        return new JsonResponse(
            array(
                'action' => 'pending',
            )
        );
    }
}
