<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Subscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class ResponseSubscriber implements EventSubscriberInterface
{

	public static function getSubscribedEvents(): array
	{
		return array(
			KernelEvents::RESPONSE => 'onKernelResponse',
		);
	}

	public function onKernelResponse(ResponseEvent $event): void
	{
		$response = $event->getResponse();
		$request  = $event->getRequest();
		if (strpos($request->getRequestUri(), '_wdt') === false) {
			$origin = $request->headers->get('origin');
			if ($origin) {
				$response->headers->set('Access-Control-Allow-Origin', $origin);
				$response->headers->set('Content-Security-Policy', "frame-ancestors 'self' $origin");
			}
			$response->headers->set('Access-Control-Allow-Credentials', 'true');
			$response->headers->set('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
			$response->headers->set('Access-Control-Allow-Headers', 'Content-Type, Authorization');

			$response->headers->set('X-Frame-Options', 'SAMEORIGIN');

			if ($request->getSession()->getId()) {
				$newCookie = new Cookie(
					$request->getSession()->getName(),
					$request->getSession()->getId(),
					$request->getSession()->getMetadataBag()->getLifetime(),
					'/',
					$request->getHost(),
					true,
					false,
					false,
					Cookie::SAMESITE_NONE
				);
				$response->headers->setCookie($newCookie);
			}
		}
	}
}
